#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

AAGroup = {
	2:['IVMLFWYC', 'ARNDQEGHKPST'],
	3:['IVLMFWC', 'YA', 'RNDQEGHKPST'],
	4:['IVLMFW', 'C', 'YA', 'RNDQEGHKPST'],
	5:['IVLMFW', 'C', 'YA', 'G', 'RNDQEHKPST'],
	6:['IVLMF', 'WY', 'C', 'AH', 'G', 'RNDQEKPST'],
	7:['IVLMF', 'WY', 'C', 'AH', 'GP', 'R', 'NDQEKST'],
	8:['IVLMF', 'WY', 'C', 'A', 'G', 'R', 'Q', 'NDEHKPST'],
	9:['IVLMF', 'WY', 'C', 'A', 'G', 'P', 'H', 'K', 'RNDQEST'],
	10:['IVLM', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'RN', 'DQEKPST'],
	11:['IVLMF', 'W', 'Y', 'C', 'A', 'H', 'G', 'R', 'N', 'Q', 'DEKPST'],
	12:['IVLM', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'T', 'RDEKPS'],
	13:['IVLM', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'P', 'R', 'DEKST'],
	14:['IVLM', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'P', 'R', 'K', 'DEST'],
	15:['IVLM', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'P', 'R', 'K', 'D', 'EST'],
	16:['IVLM', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'P', 'R', 'K', 'S', 'T', 'DE'],
	17:['IVL', 'M', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'P', 'R', 'K', 'S', 'T', 'DE'],
	18:['IVL', 'M', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'P', 'R', 'K', 'S', 'T', 'D', 'E'],
	20:['I', 'V', 'L', 'M', 'F', 'W', 'Y', 'C', 'A', 'H', 'G', 'N', 'Q', 'P', 'R', 'K', 'S', 'T', 'D', 'E'],
}
'''
for key in AAGroup:
	aa = set(list(''.join(AAGroup[key])))
	if len(aa) != 20:
		print(key, 1)
	if key != len(AAGroup[key]):
		print(key, 2)
	print(''.join(AAGroup[key]))
'''

def gapModel(fastas, myDict, gDict, gNames, ktuple, glValue):
	encodings = []
	header = ['#']

	if ktuple == 1:
		header = header + [g + '_gap' + str(glValue) for g in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence), glValue+1):
				numDict[gDict[myDict[sequence[j]]]] = numDict.get(gDict[myDict[sequence[j]]], 0) + 1

			for g in gNames:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 2:
		header = header + [g1 + '_' + g2 + '_gap' + str(glValue) for g1 in gNames for g2 in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence), glValue + 1):
				if j+1 < len(sequence):
					numDict[gDict[myDict[sequence[j]]]+'_'+gDict[myDict[sequence[j+1]]]] = numDict.get(gDict[myDict[sequence[j]]]+'_'+gDict[myDict[sequence[j+1]]], 0) + 1

			for g in [g1+'_'+g2 for g1 in gNames for g2 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 3:
		header = header + [g1 + '_' + g2 + '_' + g3 + '_gap' + str(glValue) for g1 in gNames for g2 in gNames for g3 in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence), glValue + 1):
				if j + 1 < len(sequence) and j + 2 < len(sequence):
					numDict[gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + 1]]] + '_' + gDict[myDict[sequence[j + 2]]]] = numDict.get(gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + 1]]] + '_' + gDict[myDict[sequence[j + 2]]], 0) + 1

			for g in [g1 + '_' + g2 + '_' +g3 for g1 in gNames for g2 in gNames for g3 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	return encodings

def lambdaModel(fastas, myDict, gDict, gNames, ktuple, glValue):
	if glValue == 0:
		print('Warning: the lambda value should not be zero in "lambda correlation" model')
		return 0

	encodings = []
	header = ['#']

	if ktuple == 1:
		header = header + [g + '_LC' + str(glValue) for g in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence)):
				numDict[gDict[myDict[sequence[j]]]] = numDict.get(gDict[myDict[sequence[j]]], 0) + 1

			for g in gNames:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 2:
		header = header + [g1 + '_' + g2 + '_LC' + str(glValue) for g1 in gNames for g2 in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence)):
				if j + glValue < len(sequence):
					numDict[gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]]] = numDict.get(
						gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]], 0) + 1

			for g in [g1 + '_' + g2 for g1 in gNames for g2 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 3:
		header = header + [g1 + '_' + g2 + '_' + g3 + '_LC' + str(glValue) for g1 in gNames for g2 in gNames for g3 in
						   gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence)):
				if j + glValue < len(sequence) and j + 2*glValue < len(sequence):
					numDict[gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]] + '_' + gDict[
						myDict[sequence[j + 2*glValue]]]] = numDict.get(
						gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]] + '_' + gDict[
							myDict[sequence[j + 2*glValue]]], 0) + 1

			for g in [g1 + '_' + g2 + '_' + g3 for g1 in gNames for g2 in gNames for g3 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	return encodings

def type1(fastas, subtype, raactype, ktuple, glValue):
	if raactype not in AAGroup:
		print('Error: the "--raactype" value is not correct.')
		return 0

	# index each amino acids to their group
	myDict = {}
	for i in range(len(AAGroup[raactype])):
		for aa in AAGroup[raactype][i]:
			myDict[aa] = i

	gDict = {}
	gNames = []
	for i in range(len(AAGroup[raactype])):
		gDict[i] = 'T12.G.'+str(i+1)
		gNames.append('T12.G.'+str(i+1))

	encodings = []
	if subtype == 'g-gap':
		encodings = gapModel(fastas, myDict, gDict, gNames, ktuple, glValue)
	else:
		encodings = lambdaModel(fastas, myDict, gDict, gNames, ktuple, glValue)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import argparse
import re
from codes import *

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.",
									 description="Generating various numerical representation schemes for protein sequences")
	parser.add_argument("--file", required=True, help="input fasta file")
	parser.add_argument("--type", required=True,
						choices=['AAC', 'EAAC', 'CKSAAP', 'DPC', 'DDE', 'TPC', 'BINARY',
								 'GAAC', 'EGAAC', 'CKSAAGP', 'GDPC', 'GTPC',
								 'AAINDEX', 'ZSCALE', 'BLOSUM62',
								 'NMBroto', 'Moran', 'Geary',
								 'CTDC', 'CTDT', 'CTDD',
								 'CTriad', 'KSCTriad',
								 'SOCNumber', 'QSOrder',
								 'PAAC', 'APAAC',
								 'KNNprotein', 'KNNpeptide',
								 'PSSM', 'SSEC', 'SSEB', 'Disorder', 'DisorderC', 'DisorderB', 'ASA', 'TA'
								 ],
						help="the encoding type")
	parser.add_argument("--path", dest='filePath',
						help="data file path used for 'PSSM', 'SSEB(C)', 'Disorder(BC)', 'ASA' and 'TA' encodings")
	parser.add_argument("--train", dest='trainFile',
						help="training file in fasta format only used for 'KNNprotein' or 'KNNpeptide' encodings")
	parser.add_argument("--label", dest='labelFile',
						help="sample label file only used for 'KNNprotein' or 'KNNpeptide' encodings")
	parser.add_argument("--order", dest='order',
						choices=['alphabetically', 'polarity', 'sideChainVolume', 'userDefined'],
						help="output order for of Amino Acid Composition (i.e. AAC, EAAC, CKSAAP, DPC, DDE, TPC) descriptors")
	parser.add_argument("--userDefinedOrder", dest='userDefinedOrder',
						help="user defined output order for of Amino Acid Composition (i.e. AAC, EAAC, CKSAAP, DPC, DDE, TPC) descriptors")
	parser.add_argument("--out", dest='outFile',
						help="the generated descriptor file")
	args = parser.parse_args()
	fastas = readFasta.readFasta(args.file)
	userDefinedOrder = args.userDefinedOrder if args.userDefinedOrder != None else 'ACDEFGHIKLMNPQRSTVWY'
	userDefinedOrder = re.sub('[^ACDEFGHIKLMNPQRSTVWY]', '', userDefinedOrder)
	if len(userDefinedOrder) != 20:
		userDefinedOrder = 'ACDEFGHIKLMNPQRSTVWY'
	myAAorder = {
		'alphabetically': 'ACDEFGHIKLMNPQRSTVWY',
		'polarity': 'DENKRQHSGTAPYVMCWIFL',
		'sideChainVolume': 'GASDPCTNEVHQILMKRFYW',
		'userDefined': userDefinedOrder
	}
	myOrder = myAAorder[args.order] if args.order != None else 'ACDEFGHIKLMNPQRSTVWY'
	kw = {'path': args.filePath, 'train': args.trainFile, 'label': args.labelFile, 'order': myOrder}

	myFun = args.type + '.' + args.type + '(fastas, **kw)'
	#print('Descriptor type: ' + args.type)
	encodings = eval(myFun)
	outFile = args.outFile if args.outFile != None else 'encoding.tsv'
	saveCode.savetsv(encodings, outFile)



# -*- coding: utf-8 -*-
"""
Created on Tue May 26 16:39:12 2020

@author: Sameitos
"""


import os, sys
import numpy as np
from sklearn.model_selection import RandomizedSearchCV, RepeatedKFold, PredefinedSplit
import pickle

import warnings
warnings.filterwarnings("ignore")


class regressors(object):


    def __init__(self,path):
        
        """
        Description: In class,6 different machine learning methods for regression 
                are introduced. Their hyperparameters are tuned by
                RandomizedSearchCV and all methods return only their hyperparameters 
                that give the best accoring to cvthat is created by RepeatedStraitKFold.
    
        Parameters:
            path: A destination point where model is saved. 
            X_train: Feature matrix
            y_train: (default = None), Label matrix, type = {list, numpy array}
            X_valid: (default = None), Validation Set, type = {list,numpy array}
            y_valid: (default = None), Validation Label, type = {list,numpy array}
        """
        
        self.path = path
        self.parameters = None
        self.n_jobs = -1
        self.random_state = 0
      
    
    def get_best_model(self, model, X_train, y_train,X_valid, y_valid):

        if X_valid is None: 
            
            cv = RepeatedKFold(n_splits=10,n_repeats = 5,random_state= self.random_state)
            
        else:
            
            if y_valid is None:
                raise ValueError(f'True label data for validation set cannot be None')
            
            X_train = list(X_train)
            y_train = list(y_train)
            
            len_tra, len_val = len(X_train),len(X_valid)
            
            X_train.extend(list(X_valid))
            y_train.extend(list(y_valid))
            
            test_fold = [0 if x in np.arange(len_tra) else -1 for x in np.arange(len_tra+len_val)]
            cv = PredefinedSplit(test_fold)


        clf = RandomizedSearchCV(model,self.parameters,n_iter = 10,
                                     n_jobs=self.n_jobs, cv = cv,
                                     scoring="f1")

        
        if y_train is not None:
            clf.fit(X_train,y_train)
        else:
            clf.fit(X_train)
        best_model = clf.best_estimator_

        if self.path is not None:

            with open(self.path, 'wb') as f:
                pickle.dump(best_model,f)

        return best_model


    def linear_regression(self,X_train,y_train,X_valid,y_valid):

        from sklearn.linear_model import LinearRegression
        from .hyperparameters import rgr_linear_regression_params as lrp
        
        self.parameters = lrp
        model = LinearRegression()
        return self.get_best_model(model,X_train,y_train,X_valid,y_valid)

    def SVM(self,X_train,y_train,X_valid,y_valid):
        from sklearn.svm import SVR
        from .hyperparameters import rgr_svm_params as svmp
        
        self.parameters = svmp
        model = SVR()
        
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)
        
    def random_forest(self,X_train,y_train,X_valid,y_valid):
        from sklearn.ensemble import RandomForestRegressor
        from .hyperparameters import rgr_random_forest_params as rfp
        
        self.parameters = rfp
        model = RandomForestRegressor()   
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)

    def MLP(self,X_train,y_train,X_valid,y_valid):
        
        from sklearn.neural_network import MLPRegressor
        from .hyperparameters import rgr_mlp_params as mlpp
        
        self.parameters = mlpp
        model = MLPRegressor()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)
    
    def decision_tree(self,X_train,y_train,X_valid,y_valid):
        from sklearn.tree import DecisionTreeRegressor
        from .hyperparameters import rgr_decision_tree_params as dtp
        
        self.parameters = dtp
        model = DecisionTreeRegressor()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)

    def gradient_boosting(self,X_train,y_train,X_valid,y_valid):
        from sklearn.ensemble import GradientBoostingRegressor as GBR
        from .hyperparameters import rgr_gradient_boosting_params as gbp

        self.parameters = gbp
        model = GBR()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)


def regression_methods(X_train,ml_type = "SVM", y_train = None ,X_valid = None,y_valid = None, path = None):
    
    """
    Description: 
        Selecting classification method and apply it to the data to train
    
    Parameters:
        ml_type: {'linear_reg','SVM','random_forest','MLP',
                'naive_bayes', decision_tree',gradient_boosting'}, default = "SVM",
                Type of machine learning algorithm.
    """
    
    if path is not None:
        if os.path.isfile(path):
            print(f'Model path {path} is already exist.'
                  f'To not lose model please provide new model path name or leave path as None')
            sys.exit(1)
            
            
    if set(y_train) == {1,-1} or set(y_train) == {1,0}:
        raise ValueError('Data must be continous not binary')

    r = regressors(path)
    
    machine_methods = {
                        'linear_reg':r.linear_regression,
                        'SVM':r.SVM,
                        'random_forest':r.random_forest,
                        'MLP':r.MLP,
                        'decision_tree':r.decision_tree,
                        'gradient_boosting':r.gradient_boosting
                    }   

    machine_methods[ml_type](X_train = X_train,y_train = y_train,X_valid = X_valid,y_valid = y_valid)

    return path






















#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys
from collections import namedtuple

MatrixCell = namedtuple('MatrixCell', 'score h v')


def print_matrix(m, seq):
    """
    Printing sequences and matrix
    :param m: [][]int
    :param seq: [2]string
    :return:
    """
    print('{:10}'.format("")),
    for char in seq[1]:
        print('{:4}'.format(char)),
    print
    for i in range(len(m)):
        if not i:
            print('{:2}'.format("")),
        else:
            print('{:2}'.format(seq[0][i - 1])),
        for j in range(len(m[i])):
            print('{:4}'.format(m[i][j].score)),
        print

def print_matrix_h(m, seq):
    """
    Printing sequences and matrix
    :param m: [][]int
    :param seq: [2]string
    :return:
    """
    print('{:10}'.format("")),
    for char in seq[1]:
        print('{:4}'.format(char)),
    print
    for i in range(len(m)):
        if not i:
            print('{:2}'.format("")),
        else:
            print('{:2}'.format(seq[0][i - 1])),
        for j in range(len(m[i])):
            print('{:4}'.format(m[i][j].h)),
        print

def print_matrix_v(m, seq):
    """
    Printing sequences and matrix
    :param m: [][]int
    :param seq: [2]string
    :return:
    """
    print('{:10}'.format("")),
    for char in seq[1]:
        print('{:4}'.format(char)),
    print
    for i in range(len(m)):
        if not i:
            print('{:2}'.format("")),
        else:
            print('{:2}'.format(seq[0][i - 1])),
        for j in range(len(m[i])):
            print('{:4}'.format(m[i][j].v)),
        print


def gap_line(gap, length):
    if length > 0:
        return gap[0] + (length - 1) * gap[1]
    else:
        return 0


def matrix_filling_NW(seq, s_matrix, gap):
    """
    Filling matrix according to Needleman-Wunsch algorithm
    :param seq: [2]string
    :param s_matrix: dict( char -> dict( char -> int))
    :param gap: [2]int
    :return:  [2]string, int
    """
    f_matrix = []  # Crating F-matrix, init with 0s
    neg_inf = float('-inf')
    for x in range(len(seq[0]) + 1):
        f_matrix.append([MatrixCell(0, neg_inf, neg_inf) for x in range(len(seq[1]) + 1)])

    for i in range(1, len(f_matrix)):  # Filling first row and column
        f_matrix[i][0] = MatrixCell(gap_line(gap, i), neg_inf, gap_line(gap, i))
    for j in range(1, len(f_matrix[0])):
        f_matrix[0][j] = MatrixCell(gap_line(gap, j), gap_line(gap, j), neg_inf)

    for i in range(1, len(f_matrix)):  # Filling all other cells
        #sys.stdout.write('-')
        for j in range(1, len(f_matrix[i])):
            d = f_matrix[i - 1][j - 1].score + s_matrix[seq[0][i - 1]][seq[1][j - 1]]
            h = max(f_matrix[i][j - 1].score + gap[0], f_matrix[i][j - 1].h + gap[1])
            v = max(f_matrix[i - 1][j].score + gap[0], f_matrix[i - 1][j].v + gap[1])
            score = max(d, h, v)  # Choosing max of variants
            f_matrix[i][j] = MatrixCell(score, h, v)
    #print('\nF-Matrix is built.')
    return result_seq(f_matrix, seq, s_matrix, gap)


def result_seq(f_matrix, seq, s_matrix, gap):
    """
    Finding trace in F matrix and getting resulting sequences
    :param f_matrix: [][]MatrixCell
    :param seq: [2]string
    :param s_matrix: dict( char -> dict( char -> int))
    :param gap: [2]int
    :return: [2]string, int
    """
    res1 = ""
    res2 = ""
    i = len(seq[0])
    j = len(seq[1])
    while i > 0 or j > 0:
        #sys.stdout.write('-')
        if i > 0 and j > 0 and f_matrix[i][j].score == f_matrix[i - 1][j - 1].score + s_matrix[seq[0][i - 1]][seq[1][j - 1]]:
            i -= 1
            j -= 1
            res1 = seq[0][i] + res1
            res2 = seq[1][j] + res2
        elif i > 0 and f_matrix[i][j].score == f_matrix[i][j].v:
            i -= 1
            res1 = seq[0][i] + res1
            res2 = "-" + res2
        elif j > 0 and f_matrix[i][j].score == f_matrix[i][j].h:
            j -= 1
            res1 = "-" + res1
            res2 = seq[1][j] + res2
    #print('\nSequences are found.')
    return res1, res2, f_matrix.pop().pop().score



#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import readFasta
import saveCode

USAGE = """
USAGE:
	python CTDTClass.py input.fasta output amino_acids_group_1 amino_acids_group_2 ... amino_acids_group_N

	input.fasta:                 the input protein sequence file in fasta format.	
	output:                      the encoding file.
	amino_acids_group_x          the amino acids groups.

EXAMPLE:
	python CTDTClass.py example/test-protein.txt CTDTClass.tsv RKEDQN GASTPHY CLVIMFW
"""

def CTDTClass(fastas, groups):
	encodings = []
	header = ['#']
	myOrder = []
	for g in range(len(groups) - 1):
		for g1 in range(g+1, len(groups)):
			header.append('Tr.%d%d%d%d' %(g+1, g1+1, g1+1, g+1))
			myOrder.append('%d%d%d%d' % (g + 1, g1 + 1, g1 + 1, g + 1))
	encodings.append(header)

	myDict = {}
	for g in range(len(groups)):
		for aa in groups[g]:
			myDict[aa] = g + 1

	myValues = list(set(myDict.values()))
	myKey = {}
	for v in range(len(myValues)):
		for v1 in range(v, len(myValues)):
			myKey['%d%d' % (myValues[v], myValues[v1])] = '%d%d%d%d' % (
			myValues[v], myValues[v1], myValues[v1], myValues[v])
			myKey['%d%d' % (myValues[v1], myValues[v])] = '%d%d%d%d' % (
			myValues[v], myValues[v1], myValues[v1], myValues[v])

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		aaPair = [sequence[j:j + 2] for j in range(len(sequence) - 1)]

		myCount = {}
		for pair in aaPair:
			tmp = myKey['%d%d' %(myDict[pair[0]], myDict[pair[1]])]
			if tmp in myCount:
				myCount[tmp] = myCount[tmp] + 1
			else:
				myCount[tmp] = 1

		for key in myOrder:
			if key in myCount:
				code.append(myCount[key] / len(aaPair))
			else:
				code.append(0)
		encodings.append(code)
	return encodings

if __name__ == '__main__':
	if len(sys.argv) < 5:
		print(USAGE)
		sys.exit(1)

	groups = sys.argv[3:]
	myStr = ''.join(groups)
	myStr = re.sub('[^ACDEFGHIKLMNPQRSTVWY]', '', myStr)
	if len(myStr) != 20 or len(set(myStr)) != 20:
		print('\nERROR: The amino acid must be no-repeat in each groups and the sum is 20!\n')
	fastas = readFasta.readFasta(sys.argv[1])
	encodings = CTDTClass(fastas, groups)
	saveCode.savetsv(encodings, sys.argv[2])


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
pPath = re.sub(r'scripts$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
from codes import readFasta
import argparse


def generateDisorder(fastas, outDir, vsl2):
	"""
	Generate predicted protein disorder information by bysing VSL2 software.

	Parameters
	----------
	file : file
		the file, which include the protein sequences in fasta format.
	
	vsl2: the VSL2 software path
		VSL2 software path, which could be download at: http://www.dabi.temple.edu/disprot/download/VSL2.tar.gz

	Returns
	-------
	a string: 
		A directory name, which include the predicted protein disorder information.
	"""

	if os.path.exists(outDir) == False:
		os.mkdir(outDir)

	for i in fastas:
		name, sequence = re.sub('\|', '', i[0]), i[1]
		with open(name + '.txt', 'w') as f:
			f.write(sequence + '\n')
		myCmd = 'java -jar ' + vsl2  + ' -s:' + name + '.txt >' + outDir + '/' + name + '.dis'
		if os.path.exists(outDir + '/' + name + '.dis') == False:
			os.system(myCmd)
	return outDir

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.", description="generate protein secondary structure profile")
	parser.add_argument("--file", required=True, help="protein sequence file in fasta format")
	parser.add_argument("--vsl2", help="the path of vsl2 program")
	args = parser.parse_args()

	vsl2 = args.vsl2 if args.vsl2 != None else 'VSL2.jar'
	fastas = readFasta.readFasta(args.file)
	outputDir = generateDisorder(fastas, 'out', vsl2)
	print('The predicted disorder are stored in: ' + outputDir)


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

AAGroup = {
	2:['AGSPDEQNHTKRMILFYVC', 'W'],
	3:['AGSPDEQNHTKRMILFYV', 'W', 'C'],
	4:['AGSPDEQNHTKRMIV', 'W', 'YFL', 'C'],
	5:['AGSPDEQNHTKR', 'W', 'YF', 'MIVL', 'C'],
	6:['AGSP', 'DEQNHTKR', 'W', 'YF', 'MIL', 'VC'],
	7:['AGP', 'DEQNH', 'TKRMIV', 'W', 'YF', 'L', 'CS'],
	8:['AG', 'DEQN', 'TKRMIV', 'HY', 'W', 'L', 'FP', 'CS'],
	9:['AG', 'P', 'DEQN', 'TKRMI', 'HY', 'W', 'F', 'L', 'VCS'],
	10:['AG', 'P', 'DEQN', 'TKRM', 'HY', 'W', 'F', 'I', 'L', 'VCS'],
	11:['AG', 'P', 'DEQN', 'TK', 'RI', 'H', 'Y', 'W', 'F', 'ML', 'VCS'],
	12:['FAS', 'P', 'G', 'DEQ', 'NL', 'TK', 'R', 'H', 'W', 'Y', 'IM', 'VC'],
	13:['FAS', 'P', 'G', 'DEQ', 'NL', 'T', 'K', 'R', 'H', 'W', 'Y', 'IM', 'VC'],
	14:['FA', 'P', 'G', 'T', 'DE', 'QM', 'NL', 'K', 'R', 'H', 'W', 'Y', 'IV', 'CS'],
	15:['FAS', 'P', 'G', 'T', 'DE', 'Q', 'NL', 'K', 'R', 'H', 'W', 'Y', 'M', 'I', 'VC'],
	16:['FA', 'P', 'G', 'ST', 'DE', 'Q', 'N', 'K', 'R', 'H', 'W', 'Y', 'M', 'L', 'I', 'VC'],
	17:['FA', 'P', 'G', 'S', 'T', 'DE', 'Q', 'N', 'K', 'R', 'H', 'W', 'Y', 'M', 'L', 'I', 'VC'],
	18:['FA', 'P', 'G', 'S', 'T', 'DE', 'Q', 'N', 'K', 'R', 'H', 'W', 'Y', 'M', 'L', 'I', 'V', 'C'],
	19:['FA', 'P', 'G', 'S', 'T', 'D', 'E', 'Q', 'N', 'K', 'R', 'H', 'W', 'Y', 'M', 'L', 'I', 'V', 'C'],
	20:['F', 'A', 'P', 'G', 'S', 'T', 'D', 'E', 'Q', 'N', 'K', 'R', 'H', 'W', 'Y', 'M', 'L', 'I', 'V', 'C'],
}

'''
for key in AAGroup:
	aa = set(list(''.join(AAGroup[key])))
	if len(aa) != 20:
		print(key, 1)
	if key != len(AAGroup[key]):
		print(key, 2)
	print(''.join(AAGroup[key]))
'''

def gapModel(fastas, myDict, gDict, gNames, ktuple, glValue):
	encodings = []
	header = ['#']

	if ktuple == 1:
		header = header + [g + '_gap' + str(glValue) for g in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence), glValue+1):
				numDict[gDict[myDict[sequence[j]]]] = numDict.get(gDict[myDict[sequence[j]]], 0) + 1

			for g in gNames:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 2:
		header = header + [g1 + '_' + g2 + '_gap' + str(glValue) for g1 in gNames for g2 in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence), glValue + 1):
				if j+1 < len(sequence):
					numDict[gDict[myDict[sequence[j]]]+'_'+gDict[myDict[sequence[j+1]]]] = numDict.get(gDict[myDict[sequence[j]]]+'_'+gDict[myDict[sequence[j+1]]], 0) + 1

			for g in [g1+'_'+g2 for g1 in gNames for g2 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 3:
		header = header + [g1 + '_' + g2 + '_' + g3 + '_gap' + str(glValue) for g1 in gNames for g2 in gNames for g3 in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence), glValue + 1):
				if j + 1 < len(sequence) and j + 2 < len(sequence):
					numDict[gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + 1]]] + '_' + gDict[myDict[sequence[j + 2]]]] = numDict.get(gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + 1]]] + '_' + gDict[myDict[sequence[j + 2]]], 0) + 1

			for g in [g1 + '_' + g2 + '_' +g3 for g1 in gNames for g2 in gNames for g3 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	return encodings

def lambdaModel(fastas, myDict, gDict, gNames, ktuple, glValue):
	if glValue == 0:
		print('Warning: the lambda value should not be zero in "lambda correlation" model')
		return 0

	encodings = []
	header = ['#']

	if ktuple == 1:
		header = header + [g + '_LC' + str(glValue) for g in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence)):
				numDict[gDict[myDict[sequence[j]]]] = numDict.get(gDict[myDict[sequence[j]]], 0) + 1

			for g in gNames:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 2:
		header = header + [g1 + '_' + g2 + '_LC' + str(glValue) for g1 in gNames for g2 in gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence)):
				if j + glValue < len(sequence):
					numDict[gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]]] = numDict.get(
						gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]], 0) + 1

			for g in [g1 + '_' + g2 for g1 in gNames for g2 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	if ktuple == 3:
		header = header + [g1 + '_' + g2 + '_' + g3 + '_LC' + str(glValue) for g1 in gNames for g2 in gNames for g3 in
						   gNames]
		encodings.append(header)
		for i in fastas:
			name, sequence = i[0], re.sub('-', '', i[1])
			code = [name]
			numDict = {}
			for j in range(0, len(sequence)):
				if j + glValue < len(sequence) and j + 2*glValue < len(sequence):
					numDict[gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]] + '_' + gDict[
						myDict[sequence[j + 2*glValue]]]] = numDict.get(
						gDict[myDict[sequence[j]]] + '_' + gDict[myDict[sequence[j + glValue]]] + '_' + gDict[
							myDict[sequence[j + 2*glValue]]], 0) + 1

			for g in [g1 + '_' + g2 + '_' + g3 for g1 in gNames for g2 in gNames for g3 in gNames]:
				code.append(numDict.get(g, 0))
			encodings.append(code)

	return encodings

def type1(fastas, subtype, raactype, ktuple, glValue):
	if raactype not in AAGroup:
		print('Error: the "--raactype" value is not correct.')
		return 0

	# index each amino acids to their group
	myDict = {}
	for i in range(len(AAGroup[raactype])):
		for aa in AAGroup[raactype][i]:
			myDict[aa] = i

	gDict = {}
	gNames = []
	for i in range(len(AAGroup[raactype])):
		gDict[i] = 'T3A.G.'+str(i+1)
		gNames.append('T3A.G.'+str(i+1))

	encodings = []
	if subtype == 'g-gap':
		encodings = gapModel(fastas, myDict, gDict, gNames, ktuple, glValue)
	else:
		encodings = lambdaModel(fastas, myDict, gDict, gNames, ktuple, glValue)

	return encodings


# -*- coding: utf-8 -*-
"""
Created on Sat Aug  7 23:00:39 2021

@author: Sameitos
"""

from profab.utils.imp_split_form import _classif_form_table, _rgr_form_table, multiform_table

def form_table(scores, learning_method = 'classif',path = 'score_path.csv'):

    '''
    Description:
        This function saves performance results to a .csv file.
    Parameters:
        scores: A dictionary that includes set and their scores
        learning_method: {'classif,'rgr}, default = 'classif', Type of
                        learnign method classification or regression
        path: default = 'score_path.csv', A destination where scores are 
                        saved. It must be .csv file.
    '''

    form_methods = {'classif':_classif_form_table,'rgr':_rgr_form_table}
    form_methods[learning_method](scores = scores, score_path = path)    
    
def multiple_form_table(score_dict, score_path = 'score_path.csv'):
    
    '''
    Description:
        This function is automatically forming score table
        by multiple input. No suitable for single input.
    Parameters:
        score_dict: A dict of scores includes results of multiple data,
        score_path: 'score_path.csv', A destination where scores are 
                        saved. It must be .csv file.
    '''
    
    multiform_table(score_dict, score_path)
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re
from collections import Counter

def GAAC(fastas, **kw):
	group = {
		'alphatic': 'GAVLMI',
		'aromatic': 'FYW',
		'postivecharge': 'KRH',
		'negativecharge': 'DE',
		'uncharge': 'STCPNQ'
	}

	groupKey = group.keys()

	encodings = []
	header = ['#']
	for key in groupKey:
		header.append(key)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		count = Counter(sequence)
		myDict = {}
		for key in groupKey:
			for aa in group[key]:
				myDict[key] = myDict.get(key, 0) + count[aa]

		for key in groupKey:
			code.append(myDict[key]/len(sequence))
		encodings.append(code)

	return encodings


# -*- coding: utf-8 -*-
"""
Created on Fri May 22 19:01:23 2020

@author: Sameitos
"""


class ScalerNormalizer():
    """
    Description:This class has 5 methods in which normalizer stands to 
                           normalize the data. Others are for scaling
    
    
    Parameters
        train: train feature matrix
        
    Return
        scaled_train: feature matrix which function was applied
        scaler: a model to scale other dataset like test datatest
        
    """
    def normalizer(self,train,norm = 'l2'):
        #normalizer for the data
        from sklearn.preprocessing import Normalizer
        
        scaler = Normalizer(norm = norm)
        scaled_train = scaler.fit_transform(train)
        return scaled_train,scaler
  

      
    def standard_scaler(self,train):
        #Standard scaler or z-score scaler on any data 
        from sklearn.preprocessing import StandardScaler
        
        scaler = StandardScaler()
        scaled_train = scaler.fit_transform(train)
        return scaled_train,scaler
        

        
    def maxabs_scaler(self,train):
        #for the sparse data
        from sklearn.preprocessing import MaxAbsScaler
        
        scaler = MaxAbsScaler()
        scaled_train = scaler.fit_transform(train)
        return scaled_train,scaler
        

    
    def minmax_scaler(self,train):
        #for the data has small std and is not gaussian dist
                      
        from sklearn.preprocessing import MinMaxScaler
        
        scaler = MinMaxScaler()
        scaled_train = scaler.fit_transform(train)
        return scaled_train,scaler 
        

    
    def robust_scaler(self,train):
        #for the data to eleminate outliers
        from sklearn.preprocessing import RobustScaler
        
        scaler = RobustScaler()
        scaled_train = scaler.fit_transform(train)
        return scaled_train,scaler 
        

def scale_methods(X_train,scale_type = 'standard'):
    
    '''
    Description:
        This function is to run scale module of ProFAB

    Paramters:
        -**X_train**: type = {list, numpy array}, A data to train scaling functions
        -**scale_type**: {'normalizer','standard','max_abs','min_max','robust'}, 
                        default = 'standard, determines the method to scale the data.
    Return:
        X_train: type = {list, numpy array}, Transformed data.
        scaler: A fitting function to transform other datasets.
    '''


    s = ScalerNormalizer()

    scaler_ways = {'normalizer':s.normalizer,'standard':s.standard_scaler,
                   'max_abs':s.maxabs_scaler,'min_max': s.minmax_scaler,
                   'robust': s.robust_scaler}
    
    X_train,scaler = scaler_ways[scale_type](X_train)
    
    return X_train,scaler



# -*- coding: utf-8 -*-
"""
Created on Fri Aug 27 13:06:23 2021

@author: Sameitos
"""

import numpy as np

'''
    This file to modify the parameters of machine learning algorithms w/o 
    meddling with the main functions. One can change, delete or add parameters
    with their true names. All parameters are provided for two sections "Regression" 
    and "Classification".
'''



'''
Classification Parameters for algorithms:
            logistic regression
            ridge class
            k-nearest neighbor
            support vector machine
            random_forest
            multi layer perceptron
            decision tree
            gradient boosting
'''
#Logistic Regression Parameters
cls_logistic_regression_params = dict(
            penalty = ['l2'],
            multi_class = ["auto",'ovr'],
            solver = ['newton-cg','saga', 'sag', 'lbfgs', 'liblinear'],
            C = np.linspace(0.001,100, num=100),
            max_iter = [2000]
            )

#Ridge Class Parameters
cls_ridge_class_params = dict(
            alpha = np.linspace(0.1,1,num=1000),
            fit_intercept = [False,True],
            normalize = [False,True]
            )

#k-Nearest Neighbor Parameters
cls_knn_params = dict(
            n_neighbors = np.arange(1,15),
            weights = ['uniform', 'distance'],
            metric = ['euclidean', 'manhattan', 'minkowski'],
            leaf_size = np.linspace(10,100,num = 20),
            algorithm = ['ball_tree', 'kd_tree']
            )

#Support Vector Machine Parameters
cls_svm_params = dict(
            kernel = ["linear","poly","rbf","sigmoid"],
            C = np.linspace(0.5,50,200),
            gamma = [10**x for x in np.linspace(-3,0,num=50)],
            max_iter = [2500]
            )

#Random Forest Parameters
cls_random_forest_params = dict(
            n_estimators = [int(i) for i in np.linspace(10,50,num=40)],
            max_features = ["auto","sqrt","log2"],
            bootstrap = [True, False],
            min_samples_split = np.linspace(0.09, 1.0, 50, endpoint=True),
            ccp_alpha = [10**x for x in np.linspace(-3.1,-1.5,num=100)]
            )

#Multilayer Perceptron Parameters
cls_mlp_params = dict(
            hidden_layer_sizes = ([int(x) for x in np.arange(100,300)],[int(x) for x in np.arange(1,5)]),
            activation = ['logistic', 'tanh', 'relu'],
            solver = ['sgd','adam'],
            alpha = np.linspace(0.0001,0.001,num = 10),
            batch_size = [100,150],
            learning_rate = ['constant', 'invscaling', 'adaptive'],
            max_iter = [1000]
            )

#Decision Tree Parameters
cls_decision_tree =dict(
            max_features = ["auto", "sqrt", "log2"],
            #max_depth = np.arange(1,32),
            #min_samples_leaf = np.arange(10,50),
            criterion = ['gini','entropy'],
            ccp_alpha = [10**x for x in np.linspace(-3.5,-1.5,num=50)]
            )

#Gradient Boosting Parameters
cls_gradient_boosting = dict(
            loss = ['deviance','exponential'],
            learning_rate = np.linspace(0.01,0.15,num = 10),
            n_estimators = np.arange(100,150),
            criterion = ['mse','friedman_mse'],
            #max_depth = np.arange(3,10),
            ccp_alpha = [10**x for x in np.linspace(-3.4,-1.5,num=50)]
            )
            

'''
Regression parameters for algorithms:
            linear regression
            support vector machine
            random forest
            multilayer perceptron
            decision tree
            gradient boosting
'''
#Linear Regression Parameters
rgr_linear_regression_params = dict(
            fit_intercept = True,
            positive = True
            )

#Support Vector Machine Parameters
rgr_svm_params = dict(
            kernel = ["poly","rbf","sigmoid",'sigmoid'],
            C = np.linspace(0.1,50,100),
            gamma = [10**x for x in np.arange(-5,5,dtype = float)],
            max_iter = np.arange(50,200)
            )

#Random Forest Parameters
rgr_random_forest_params = dict(
            n_estimators = [int(i) for i in np.linspace(10,50,num=40)],
            max_features = ["auto","sqrt","log2"],
            bootstrap = [True, False],
            max_depth = np.arange(1,32),
            min_samples_split = np.linspace(0.1, 1.0, 9, endpoint=True)
            )

#Multilayer Perceptron Parameters
rgr_mlp_params = dict(
            hidden_layer_sizes = ([int(x) for x in np.arange(100,300)],[int(x) for x in np.arange(1,5)]),
            activation = ['logistic', 'tanh', 'relu'],
            solver = ['sgd','adam'],
            alpha = np.linspace(0.0001,0.001,num = 10),
            batch_size = [100,150],
            learning_rate = ['constant', 'invscaling', 'adaptive'],
            max_iter = np.arange(20,50)
            )

#Decision Tree Parameters
rgr_decision_tree_params = dict(
            criterion = ['mse', 'friedman_mse', 'mae'],
            max_features = ["auto", "sqrt", "log2"],
            max_depth = np.arange(1,32),
            min_samples_split = np.linspace(0.1, 1.0, 9, endpoint=True),
            max_leaf_nodes = np.arange(2,20)
            )

#Gradient Boosting Paramters
rgr_gradient_boosting_params = dict(
            loss = ['ls','lad','huber','quantile'],
            learning_rate = np.linspace(0.01,0.15,num = 10),
            n_estimators = np.arange(75,150),
            criterion = ['mse','mae','friedman_mse'],
            max_depth = np.arange(3,10),
            subsample = np.linspace(0.5,1.5,10,endpoint=True)
            )




























#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
from sklearn.cluster import DBSCAN
from sklearn.preprocessing import StandardScaler

def dbscan(encodings, **kw):
	if kw['sof'] == 'sample':
		encodings = np.array(encodings)[1:]
	else:
		encodings = np.array(encodings).T[1:]

	if len(encodings) < 5:
		return 0, 'sample number should be greater than 5.'

	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	e = ''
	dataNew = StandardScaler().fit_transform(data)
	db = DBSCAN().fit(dataNew)
	core_samples_mask = np.zeros_like(db.labels_, dtype=bool)
	core_samples_mask[db.core_sample_indices_] = True
	labels = db.labels_

	res = []
	for i in range(len(data)):
		res.append([encodings[i][0], labels[i]])
	return res, e


#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
import pandas as pd
import math

binBox = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]

def calProb(array):
	myProb = {}
	myClass = set(array)
	for i in myClass:
		myProb[i] = array.count(i) / len(array)
	return myProb

def jointProb(newArray, labels):
	myJointProb = {}
	for i in range(len(labels)):
		myJointProb[str(newArray[i]) + '-' + str(labels[i])] = myJointProb.get(str(newArray[i]) + '-' + str(labels[i]), 0) + 1

	for key in myJointProb:
		myJointProb[key] = myJointProb[key] / len(labels)
	return myJointProb

def MIC(encodings, labelFile):
	features = encodings[0][1:]
	encodings = np.array(encodings)[1:]
	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	e = ''
	if shape[0] < 5 or shape[1] < 2:
		return 0, e

	with open(labelFile) as f:
		records = f.readlines()
	myDict = {}
	try:
		for i in records:
			array = i.rstrip().split() if i.strip() != '' else None
			myDict[array[0]] = int(array[1])
	except IndexError as e:
		print(e)
		return 0, e

	labels = []
	for i in encodings:
		labels.append(myDict.get(i[0], 0))

	dataShape = data.shape

	if dataShape[1] != len(features):
		print('Error: inconsistent data shape with feature number.')
		return 0, 'Error: inconsistent data shape with feature number.'

	if dataShape[0] != len(labels):
		print('Error: inconsistent data shape with sample number.')
		return 0, 'Error: inconsistent data shape with sample number.'

	sampleNumber = len(data)
	labelClass = set(labels)
	probY = calProb(labels)
	myFea = {}
	for i in range(len(features)):
		array = data[:, i]
		newArray = list(pd.cut(array, len(binBox), labels= binBox))
		binBoxClass = set(newArray)
		probX = calProb(newArray)
		probXY = jointProb(newArray, labels)
		mic = 0
		for x in probX.keys():
			for y in probY.keys():
				if str(x) + '-' + str(y) in probXY:
					mic = mic + probXY[str(x) + '-' + str(y)] * math.log(probXY[str(x) + '-' + str(y)]/(probX[x] * probY[y]), 2)
		myFea[features[i]] = mic

	res = []
	res.append(['feature', 'MIC-value'])
	for key in sorted(myFea.items(), key=lambda item:item[1], reverse=True):
		res.append([key[0], '{0:.3f}'.format(myFea[key[0]])])
	return res, e



#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
pPath = re.sub(r'scripts$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
from codes import readFasta
import argparse
dbName = '/home/zchen/Databases/uniref50/uniref50'
ncbidir = '/home/zchen/local/Tools/blast-2.2.18/bin'

def generatePSSMProfile(fastas, outDir, blastpgp, db):
	"""
	Generate PSSM file by using the blastpgp program in NCBI blast-2.2.18 package.

	Parameters
	----------
	file : file
		the file, which include the protein sequences in fasta format.

	blastpgp: string
		the path of blastpgp program.
		
	db: string 
		the uniref50 data base, which is formated by the 'formatdb' program in blast package.

	Returns
	-------
	a string: 
		A directory name, which include the predicted protein PSSM information.
	"""


	if os.path.exists(outDir) == False:
		os.mkdir(outDir)

	for i in fastas:
		name, sequence = re.sub('\|', '', i[0]), i[1]
		with open(name + '.txt', 'w') as f:
			f.write('>'+name+'\n'+sequence + '\n')
		myCmd = blastpgp + ' -i ' + name + '.txt' + ' -d ' + db + ' -b 0 -v 5000 -j 3 -h 0.001 -Q ' + outDir + '/' + name +'.pssm'
		print('Doing psiblast for protein: ' + name)
		os.system(myCmd)
		os.remove(name + '.txt')
	return outDir


if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.", description="generate PSSM profile")
	parser.add_argument("--file", required=True, help="protein sequence file in fasta format")
	parser.add_argument("--blastpgp", help="the path of NCBI psiblast program")
	parser.add_argument("--db", help="the path of unief50 database")
	args = parser.parse_args()

	blastpgp = args.blastpgp if args.blastpgp != None else ncbidir + '/blastpgp'
	db = args.db if args.db != None else dbName
	fastas = readFasta.readFasta(args.file)
	outputDir = generatePSSMProfile(fastas, 'out', blastpgp, db)
	print('The PSSM profiles are stored in directory: ' + outputDir)











#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

def Count(seq1, seq2):
	sum = 0
	for aa in seq1:
		sum = sum + seq2.count(aa)
	return sum


def CTDC(fastas, **kw):
	group1 = {
		'hydrophobicity_PRAM900101': 'RKEDQN',
		'hydrophobicity_ARGP820101': 'QSTNGDE',
		'hydrophobicity_ZIMJ680101': 'QNGSWTDERA',
		'hydrophobicity_PONP930101': 'KPDESNQT',
		'hydrophobicity_CASG920101': 'KDEQPSRNTG',
		'hydrophobicity_ENGD860101': 'RDKENQHYP',
		'hydrophobicity_FASG890101': 'KERSQD',
		'normwaalsvolume': 'GASTPDC',
		'polarity':        'LIFWCMVY',
		'polarizability':  'GASDT',
		'charge':          'KR',
		'secondarystruct': 'EALMQKRH',
		'solventaccess':   'ALFCGIVW'
	}
	group2 = {
		'hydrophobicity_PRAM900101': 'GASTPHY',
		'hydrophobicity_ARGP820101': 'RAHCKMV',
		'hydrophobicity_ZIMJ680101': 'HMCKV',
		'hydrophobicity_PONP930101': 'GRHA',
		'hydrophobicity_CASG920101': 'AHYMLV',
		'hydrophobicity_ENGD860101': 'SGTAW',
		'hydrophobicity_FASG890101': 'NTPG',
		'normwaalsvolume': 'NVEQIL',
		'polarity':        'PATGS',
		'polarizability':  'CPNVEQIL',
		'charge':          'ANCQGHILMFPSTWYV',
		'secondarystruct': 'VIYCWFT',
		'solventaccess':   'RKQEND'
	}
	group3 = {
		'hydrophobicity_PRAM900101': 'CLVIMFW',
		'hydrophobicity_ARGP820101': 'LYPFIW',
		'hydrophobicity_ZIMJ680101': 'LPFYI',
		'hydrophobicity_PONP930101': 'YMFWLCVI',
		'hydrophobicity_CASG920101': 'FIWC',
		'hydrophobicity_ENGD860101': 'CVLIMF',
		'hydrophobicity_FASG890101': 'AYHWVMFLIC',
		'normwaalsvolume': 'MHKFRYW',
		'polarity':        'HQRKNED',
		'polarizability':  'KMHFRYW',
		'charge':          'DE',
		'secondarystruct': 'GNPSD',
		'solventaccess':   'MSPTHY'
	}

	groups = [group1, group2, group3]
	property = (
	'hydrophobicity_PRAM900101', 'hydrophobicity_ARGP820101', 'hydrophobicity_ZIMJ680101', 'hydrophobicity_PONP930101',
	'hydrophobicity_CASG920101', 'hydrophobicity_ENGD860101', 'hydrophobicity_FASG890101', 'normwaalsvolume',
	'polarity', 'polarizability', 'charge', 'secondarystruct', 'solventaccess')

	encodings = []
	header = ['#']
	for p in property:
		for g in range(1, len(groups) + 1):
			header.append(p + '.G' + str(g))
	encodings.append(header)
	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		for p in property:
			c1 = Count(group1[p], sequence) / len(sequence)
			c2 = Count(group2[p], sequence) / len(sequence)
			c3 = 1 - c1 - c2
			code = code + [c1, c2, c3]
		encodings.append(code)
	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re

"""
import sys, platform

def CTriad(fastas, **kw):
	dataFile = sys.path[0] + r'\data\CTD.txt' if platform.system() == 'Windows' else sys.path[0] + '/data/CTD.txt'
	CTDIndex = {}
	with open(dataFile) as f:
		records = f.readlines()
	for i in records:
		array = i.rstrip().split() if i.rstrip() != '' else None
		CTDIndex[array[0]] = int(array[1])

	encodings = []
	header = ['#']
	for i in range(1, 344):
		header.append('CTriad.' + str(i))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name] + [0] * 343
		myDict = {}
		for j in range(len(sequence) - 2):
			key = sequence[j:j+3]
			myDict[key] = myDict[key] + 1 if key in myDict else 1
		for key in myDict:
			code[CTDIndex[key]] = code[CTDIndex[key]] + myDict[key]
		maxValue, minValue = max(code[1:]), min(code[1:])
		for j in range(1, 344):
			code[j] = (code[j] - minValue) / maxValue
		encodings.append(code)
	return encodings
"""

def CalculateKSCTriad(sequence, gap, features, AADict):
	res = []
	for g in range(gap+1):
		myDict = {}
		for f in features:
			myDict[f] = 0

		for i in range(len(sequence)):
			if i+gap+1 < len(sequence) and i+2*gap+2<len(sequence):
				fea = AADict[sequence[i]] + '.' + AADict[sequence[i+gap+1]]+'.'+AADict[sequence[i+2*gap+2]]
				myDict[fea] = myDict[fea] + 1

		maxValue, minValue = max(myDict.values()), min(myDict.values())
		for f in features:
			res.append((myDict[f] - minValue) / maxValue)

	return res

def CTriad(fastas, gap = 0, **kw):
	AAGroup = {
		'g1': 'AGV',
		'g2': 'ILFP',
		'g3': 'YMTS',
		'g4': 'HNQW',
		'g5': 'RK',
		'g6': 'DE',
		'g7': 'C'
	}

	myGroups = sorted(AAGroup.keys())

	AADict = {}
	for g in myGroups:
		for aa in AAGroup[g]:
			AADict[aa] = g

	features = [f1 + '.'+ f2 + '.' + f3 for f1 in myGroups for f2 in myGroups for f3 in myGroups]

	encodings = []
	header = ['#']
	for f in features:
		header.append(f)
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		if len(sequence) < 3:
			print('Error: for "CTriad" encoding, the input fasta sequences should be greater than 3. \n\n')
			return 0
		code = code + CalculateKSCTriad(sequence, 0, features, AADict)
		encodings.append(code)

	return encodings


# -*- coding: utf-8 -*-
"""
Created on Tue Mar  9 00:30:06 2021

@author: Sameitos
"""

import os
import re
import sys
import requests
import numpy as np
from tqdm import tqdm
from sklearn.model_selection import train_test_split
from zipfile import ZipFile

def download_data(server_path,save_path):
    
    headers=requests.head(server_path).headers
    downloadable = 'Content-Length' in headers.keys()
    if downloadable:
        response = requests.get(server_path, stream = True)
        
        total_byte= int(response.headers.get('content-length', 0)) 
        progress_bar = tqdm(total=total_byte, unit='iB', unit_scale=True)
        with open(save_path, 'wb') as file:
            for data in response.iter_content(chunk_size= 8*1024):
                progress_bar.update(len(data))
                file.write(data)
        progress_bar.close()
    else:
        print("No given dataset is available in server.")
        sys.exit(1)

def separator(X,y,ratio):
    
    if type(ratio) == float:

        return train_test_split(X,y, test_size = ratio)

    elif type(ratio) == list:

        X_train,X_test,y_train,y_test = train_test_split(X,y, test_size = ratio[0])

        X_train,X_validation,y_train,y_validation = train_test_split(X_train,y_train,
                                                                     test_size = ratio[1]/(
                                                                         1-ratio[0]))
            
        return X_train,X_test,X_validation,y_train,y_test,y_validation
    
    
def _rgr_data_import(zip_data,xf,yf,indices_file = None):
    X,y = [],[]
    with ZipFile(zip_data) as f:
        ready_indices = set([int(i.decode('utf-8').strip('\n')) for i in f.open(indices_file)])
        xff =  f.open(xf)
        yff = f.open(yf)
        if indices_file is not None:
            for k,(rowx,rowy) in enumerate(zip(xff,yff)):
                if k in ready_indices:
                    rowx = re.split(' ',rowx.strip('\n'))
                    rowx = list(np.array(rowx,dtype = 'float64'))
                    X.append(rowx)
                    y.append(rowy.strip('\n'))
    
        else:
            for rowx,rowy in zip(xff,yff):
            
                rowx = re.split(' ',rowx.strip('\n'))
                rowx = list(np.array(rowx,dtype = 'float64'))
                X.append(rowx)
                y.append(rowy.strip('\n'))
    xf.close()
    yf.close()
    
    return X,y
    

def _classif_data_import(zip_data,pos_file,neg_file, label, pos_indices = None,neg_indices = None):
    
    
    pX,nX,X,y = [],[],[],[]
    
    with ZipFile(zip_data) as f:
        pf =  f.open(pos_file)
        nf = f.open(neg_file)
        
        if pos_indices is not None and neg_indices is not None:
	        pos_idx = set([int(i.decode('utf-8').strip('\n')) for i in f.open(pos_indices)])
	        neg_idx = set([int(i.decode('utf-8').strip('\n')) for i in f.open(neg_indices)])
	        for k,(rowx) in enumerate(pf):
	            if k in pos_idx:
	                rowx = re.split('\t',rowx.decode('utf-8').strip('\n'))
	                rowx = list(np.array(rowx[1:],dtype = 'float64'))
	                if label == 'positive':
	                    pX.append(rowx)
	                if label == None:
	                    X.append(rowx)
	                    y.append(1)
	        for k,(rowx) in enumerate(nf):
	            if k in neg_idx:
	                rowx = re.split('\t',rowx.decode('utf-8').strip('\n'))
	                rowx = list(np.array(rowx[1:],dtype = 'float64'))
	                if label == 'negative':
	                    nX.append(rowx)
	                if label == None:
	                    X.append(rowx)
	                    y.append(-1)
        else:

            for rowx in pf:
                rowx = re.split('\t',rowx.decode('utf-8').strip('\n'))
                rowx = list(np.array(rowx[1:],dtype = 'float64'))
                if label == 'positive':
                    pX.append(rowx)
                    
                if label == None:
                    X.append(rowx)
                    y.append(1)
            for rowx in nf:
                rowx = re.split('\t',rowx.decode('utf-8').strip('\n'))
                rowx = list(np.array(rowx[1:],dtype = 'float64'))
                if label == 'negative':
                    nX.append(rowx)
                    
                if label == None:
                    X.append(rowx)
                    y.append(-1)
    pf.close()
    nf.close()
        
    return pX,nX,X,y

def self_data(file_name, delimiter, label, name):
        
    
    with open(file_name, 'r') as f:
            
        if label:
            
            X_pos,X_neg = [],[]
            for row in f:
                row = re.split(delimiter,row.strip())
                if name:
                    if int(row[-1]) == 1:

                        X_pos.append(row[1:-1])
                    else:
                        X_neg.append(row[1:-1])

                else:
                    if int(row[-1]) == 1:

                        X_pos.append(row[1:-1])
                    else:
                        X_neg.append(row[1:-1])
            
            return X_pos,X_neg
        else:
            X = []
            for row in f:
                row = re.split(delimiter,row.strip())
                
                if name:
                        X.append(row[1:])
                else:
                    X.append(row)
            return X


def _classif_form_table(scores, score_path = 'score_path.csv'):

    func = 'w'
    if os.path.isfile(score_path):
        print(f'File {score_path} already exists, Scores are append to old score path')
        func = 'a'
    
    if type(scores) is not dict:
        raise TypeError('Type "scores" should be dictionary')
    
    f = open(score_path,func)
    
    scores.values()
    columns = ['Set'] + list(list(scores.values())[0].keys())
    f.write(f'{",".join(columns)}\n')
    
    for sc in scores.keys():
        score = np.array([sc] + list(scores[sc].values()),dtype = str)          
        f.write(f'{",".join(score)}\n')
    
    f.write(f'\n')
    f.close()
    
def _rgr_form_table(scores, size = None, score_path = 'score_path.csv'):
    
    func = 'w'
    if os.path.isfile(score_path):
        print(f'File {score_path} already exists, Scores are append to old score path')
        func = 'a'
        

    if type(scores) is not dict:
        raise TypeError('Type "scores" should be dictionary')
    
    f = open(score_path,func)
    columns = ['Set'] + list(list(scores.values())[0].keys())[:-1] + list(
        
        list(scores.values())[0]['threshold based Metrics'].keys())
    
    f.write(f'{",".join(columns)}\n')
    
    for sc in scores.keys():   
                   
        score = np.array([sc] + list(scores[sc].values())[:-1] + list(
            scores[sc]['threshold based Metrics'].values()) ,dtype = str)
        f.write(f'{",".join(score)}\n')

    f.write(f'\n')
    f.close()

def multiform_table(score_dict, score_path):
    
    func = 'w'
    if os.path.isfile(score_path):
        print(f'File {score_path} already exists, Scores are append to old score path')
        func = 'a'
        
    f = open(score_path, func)
    
    datasets = list(score_dict.keys())
    columns = ['Dataset Name', 'Set'] + list(list(score_dict[datasets[0]].values())[0].keys())
    f.write(f'{",".join(columns)}\n')
    for data_name in datasets:
        
        f.write(f'{data_name}')
        scores = score_dict[data_name]
        
        for sc in scores.keys():
            
            score = np.array([" "] + [sc] + list(scores[sc].values()),dtype = str)          
            f.write(f'{",".join(score)}\n')
        f.write(f'\n')
    
    f.write(f'\n')
    f.close()
    
    


#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
import pandas as pd

def saveCluster(cluster, e, file='Clusters.txt'):
	with open(file, 'w') as f:
		if cluster == 0:
			f.write(str(e))
		else:
			myCluster = np.array(cluster)
			df = pd.DataFrame({'name': myCluster[:, 0], 'cluster': myCluster[:, 1]})
			mySet = set(np.array(df.cluster).tolist())
			f.write('# The sample/feature can be clustered into %d clusters:\n' %len(mySet))
			for l in sorted(mySet):
				newData = np.array(df.loc[df.loc[:, "cluster"] == l, :].name)
				f.write('Cluster_%s:\t' % l)
				for i in newData:
					f.write(i + '\t')
				f.write('\n')
			f.write('\n==============================================================\n')

			f.write('Protein/Feature\tcluster\n')
			for i in cluster:
				f.write(i[0] + '\t' + str(i[1]) + '\n')
	return None


# -*- coding: utf-8 -*-
"""
Created on Tue May 26 18:45:38 2020

@author: Sameitos
"""

import os, sys
import numpy as np
from sklearn.model_selection import RandomizedSearchCV,RepeatedStratifiedKFold, PredefinedSplit
import pickle

import warnings
warnings.filterwarnings("ignore")


class classifiers(object):

    
    def __init__(self,path):     
        
        """
        Description: In class,6 different machine learning methods for regression 
                are introduced. Their hyperparameters are tuned by
                RandomizedSearchCV and all methods return only their hyperparameters 
                that give the best accoring to cvthat is created by RepeatedStraitKFold.
    
        Parameters:
            path: A destination point where model is saved.
            X_train: Feature matrix
            y_train: (default = None), Label matrix, type = {list, numpy array}
            X_valid: (default = None), Validation Set, type = {list,numpy array}
            y_valid: (default = None), Validation Label, type = {list,numpy array}
        """
        
        self.path = path
        self.parameters = None
        self.n_jobs = -1
        self.random_state = 0

      
    def get_best_model(self, model, X_train, y_train,X_valid, y_valid):
        


        if X_valid is None: 
            
            cv = RepeatedStratifiedKFold(n_splits=10,n_repeats = 5,random_state= self.random_state)
            
        else:
            
            if y_valid is None:
                raise ValueError('True label data for validation set cannot be None')
            
            X_train = list(X_train)
            y_train = list(y_train)
            
            len_tra, len_val = len(X_train),len(X_valid)
            
            X_train.extend(list(X_valid))
            y_train.extend(list(y_valid))
            
            test_fold = [0 if x in np.arange(len_tra) else -1 for x in np.arange(len_tra+len_val)]
            cv = PredefinedSplit(test_fold)

        clf = RandomizedSearchCV(model,self.parameters,n_iter = 10,
                                     n_jobs=self.n_jobs, cv = cv,
                                     scoring="f1", random_state = self.random_state)

        if y_train is not None:
            clf.fit(X_train,y_train)
        else:
            clf.fit(X_train)

        best_model = clf.best_estimator_
        print(best_model)

        if self.path is not None:
            with open(self.path, 'wb') as f:
                pickle.dump(best_model,f)

        return best_model


    def logistic_regression(self,X_train,y_train,X_valid,y_valid):    
        from sklearn.linear_model import LogisticRegression
        from .hyperparameters import cls_logistic_regression_params as lrp

        self.parameters = lrp
        model = LogisticRegression()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)
        

    def ridge_class(self,X_train,y_train,X_valid,y_valid):
        from sklearn.linear_model import RidgeClassifier
        from .hyperparameters import cls_ridge_class_params as rcp

        self.parameters = rcp
        model = RidgeClassifier()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)

        
    def KNN(self,X_train,y_train,X_valid,y_valid):
        from sklearn.neighbors import KNeighborsClassifier
        from .hyperparameters import cls_knn_params as kp

        self.parameters  = kp
        model = KNeighborsClassifier()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)

    
    def SVM(self,X_train,y_train,X_valid,y_valid):
        from sklearn.svm import SVC
        from .hyperparameters import cls_svm_params as sp
        
        self.parameters = sp
        model = SVC()
        return  self.get_best_model(model, X_train, y_train,X_valid, y_valid)


    def random_forest(self,X_train,y_train,X_valid,y_valid):
        from sklearn.ensemble import RandomForestClassifier
        from .hyperparameters import cls_random_forest_params as rfp
        
        self.parameters = rfp
        model = RandomForestClassifier()   
        return  self.get_best_model(model, X_train, y_train,X_valid, y_valid)


    def MLP(self,X_train,y_train,X_valid,y_valid):
        from sklearn.neural_network import MLPClassifier
        from .hyperparameters import cls_mlp_params as mlpp

        self.parameters = mlpp
        model = MLPClassifier()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)


    def decision_tree(self,X_train,y_train,X_valid,y_valid):
        from sklearn.tree import DecisionTreeClassifier
        from .hyperparameters import cls_decision_tree as dtp

        self.parameters = dtp
        model = DecisionTreeClassifier()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)


    def gradient_boosting(self,X_train,y_train,X_valid,y_valid):
        from sklearn.ensemble import GradientBoostingClassifier as GBC
        from .hyperparameters import cls_gradient_boosting as gbp
        
        self.parameters = gbp
        model = GBC()
        return self.get_best_model(model, X_train, y_train,X_valid, y_valid)


    def naive_bayes(self,X_train,y_train,X_valid,y_valid):
        from sklearn.naive_bayes import GaussianNB
        
        model = GaussianNB()
        model.fit(X_train,y_train)
        with open(self.path,'wb') as f:
            pickle.dump(model, f)
        return model


def classification_methods(X_train,ml_type = 'SVM', y_train = None,
                           X_valid = None,y_valid = None,
                           path = None):
    
    """
    Description: 
        Selecting classification method and apply it to the data to train
    
    Parameters:
        ml_type: {'logistic_reg','ridge_class','KNN','SVM','random_forest','MLP',
                'naive_bayes', decision_tree',gradient_boosting'}, default = "SVM",
                Type of machine learning algorithm.
    """
    if path is not None:
        if os.path.isfile(path):
            print(f'Model path {path} is already exist.'
                  f'To not lose model please provide new model path name or leave path as None')
            sys.exit(1)
                
    
    if not set(y_train) == {1,-1} or set(y_train) == {1,0}:
    	raise ValueError(f'Data must be binary: {{1,-1}} or {{1,0}}')

    c = classifiers(path)
    
    machine_methods = {'logistic_reg':c.logistic_regression,'ridge_class':c.ridge_class,
                     'KNN':c.KNN,'SVM':c.SVM,'random_forest':c.random_forest,
                    'MLP':c.MLP,'naive_bayes':c.naive_bayes,'decision_tree':c.decision_tree,
                    'gradient_boosting':c.gradient_boosting}    

    return machine_methods[ml_type](X_train = X_train,
                                    y_train = y_train,
                                    X_valid = X_valid,
                                    y_valid = y_valid)
























#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys

def savetsv(encodings, file = 'encoding.tsv'):
	with open(file, 'w') as f:
		if encodings == 0:
			f.write('Descriptor calculation failed.')
		else:
			for i in range(len(encodings[0]) - 1):
				f.write(encodings[0][i] + '\t')
			f.write(encodings[0][-1] + '\n')
			for i in encodings[1:]:
				f.write(i[0] + '\t')
				for j in range(1, len(i) - 1):
					f.write(str(float(i[j])) + '\t')
				f.write(str(float(i[len(i)-1])) + '\n')
	return None



import numpy as np
import copy
from math import sqrt
from scipy import stats
from sklearn import preprocessing, metrics
from sklearn.metrics import confusion_matrix

def get_cindex(Y, P):
    summ = 0
    pair = 0

    for i in range(1, len(Y)):
        for j in range(0, i):
            if i is not j:
                if (Y[i] > Y[j]):
                    pair += 1
                    summ += 1 * (P[i] > P[j]) + 0.5 * (P[i] == P[j])

    if pair is not 0:
        return summ / pair
    else:
        return 0


def r_squared_error(y_obs, y_pred):
    y_obs = np.array(y_obs)
    y_pred = np.array(y_pred)
    y_obs_mean = [np.mean(y_obs) for y in y_obs]
    y_pred_mean = [np.mean(y_pred) for y in y_pred]

    mult = sum((y_pred - y_pred_mean) * (y_obs - y_obs_mean))
    mult = mult * mult

    y_obs_sq = sum((y_obs - y_obs_mean) * (y_obs - y_obs_mean))
    y_pred_sq = sum((y_pred - y_pred_mean) * (y_pred - y_pred_mean))

    return mult / float(y_obs_sq * y_pred_sq)


def get_k(y_obs, y_pred):
    y_obs = np.array(y_obs)
    y_pred = np.array(y_pred)

    return sum(y_obs * y_pred) / float(sum(y_pred * y_pred))


def squared_error_zero(y_obs, y_pred):
    k = get_k(y_obs, y_pred)

    y_obs = np.array(y_obs)
    y_pred = np.array(y_pred)
    y_obs_mean = [np.mean(y_obs) for y in y_obs]
    upp = sum((y_obs - (k * y_pred)) * (y_obs - (k * y_pred)))
    down = sum((y_obs - y_obs_mean) * (y_obs - y_obs_mean))

    return 1 - (upp / float(down))


def get_rm2(ys_orig, ys_line):
    r2 = r_squared_error(ys_orig, ys_line)
    r02 = squared_error_zero(ys_orig, ys_line)

    return r2 * (1 - np.sqrt(np.absolute((r2 * r2) - (r02 * r02))))


"""
@author: Anna Cichonska
"""



def mse(y,f):
    """
    Task:    To compute root mean squared error (RMSE)

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  mse   MSE
    """

    mse = ((y - f)**2).mean(axis=0)

    return mse

def rmse(y,f):
    """
    Task:    To compute root mean squared error (RMSE)

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  rmse   RSME
    """

    rmse = sqrt(((y - f)**2).mean(axis=0))

    return rmse


def pearson(y,f):
    """
    Task:    To compute Pearson correlation coefficient

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  rp     Pearson correlation coefficient
    """

    rp = np.corrcoef(y, f)[0,1]

    return rp


def spearman(y,f):
    """
    Task:    To compute Spearman's rank correlation coefficient

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  rs     Spearman's rank correlation coefficient
    """

    rs = stats.spearmanr(y, f)[0]

    return rs

def cl_prec_rec_f1_acc_mcc(y_true, y_pred):
    """
    Task:    To compute F1 score using the threshold of 7 M
             to binarize pKd's into true class labels.

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  f1     F1 score
    """
    performance_threshold_dict  = dict()
    precision = metrics.precision_score(y_true, y_pred)
    recall = metrics.recall_score(y_true, y_pred)
    f1_score = metrics.f1_score(y_true, y_pred)
    accuracy = metrics.accuracy_score(y_true, y_pred)
    mcc = metrics.matthews_corrcoef(y_true, y_pred)
    f05_score = 1.25*precision*recall/(0.25*precision+recall)
    tn, fp, fn, tp = confusion_matrix(y_true, y_pred).ravel()
    auc = classif_AUC(y_true,y_pred)
    auprc = classif_AUPRC(y_true, y_pred)
    
    performance_threshold_dict["Precision"] = precision
    performance_threshold_dict["Recall"] = recall
    performance_threshold_dict["F1-Score"] = f1_score
    performance_threshold_dict["F05-Score"] = f05_score
    performance_threshold_dict["Accuracy"] = accuracy
    performance_threshold_dict["MCC"] = mcc
    performance_threshold_dict["AUC"] = auc
    performance_threshold_dict["AUPRC"] = auprc
    performance_threshold_dict["TP"] = tp
    performance_threshold_dict["FP"] = fp
    performance_threshold_dict["TN"] = tn
    performance_threshold_dict["FN"] = fn
    
    return performance_threshold_dict


def reg_prec_rec_f1_acc_mcc(y,f):
    """
    Task:    To compute F1 score using the threshold of 7 M
             to binarize pKd's into true class labels.

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  f1     F1 score
    """
    # 10 uM, 1 uM, 100 nM
    str_threshold_lst = ["10uM", "1uM", "100nM", "30nM"]
    threshold_lst = [5.0, 6.0, 7.0, 7.522878745280337]
    dict_threshold = {str_threshold_lst[0]:threshold_lst[0] ,str_threshold_lst[1]:threshold_lst[1],
                      str_threshold_lst[2]:threshold_lst[2], str_threshold_lst[3]:threshold_lst[3]}

    performance_threshold_dict = dict()
    for str_thre, threshold in dict_threshold.items():
        y_binary = copy.deepcopy(y)
        y_binary = preprocessing.binarize(y_binary.reshape(1,-1), threshold, copy=False)[0]
        f_binary = copy.deepcopy(f)
        f_binary = preprocessing.binarize(f_binary.reshape(1,-1), threshold, copy=False)[0]
        precision = metrics.precision_score(y_binary, f_binary)
        recall = metrics.recall_score(y_binary, f_binary)
        f1_score = metrics.f1_score(y_binary, f_binary)
        accuracy = metrics.accuracy_score(y_binary, f_binary)
        mcc = metrics.matthews_corrcoef(y_binary, f_binary)
        performance_threshold_dict["Precision {}".format(str_thre)] = precision
        performance_threshold_dict["Recall {}".format(str_thre)] = recall
        performance_threshold_dict["F1-Score {}".format(str_thre)] = f1_score
        performance_threshold_dict["Accuracy {}".format(str_thre)] = accuracy
        performance_threshold_dict["MCC {}".format(str_thre)] = mcc

    return performance_threshold_dict

def classif_AUC(y,f):
    
    """
    Task:    To compute average area under the ROC curves (AUC)

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  avAUC   average AUC

    """
    fpr, tpr, thresholds = metrics.roc_curve(y, f, pos_label=1)
    auc = metrics.auc(fpr, tpr)
    return auc
    
def classif_AUPRC(y,f):
    
    precision, recall, thresholds = metrics.precision_recall_curve(y, f, pos_label=1)
    auprc = metrics.auc(recall, precision)


    return auprc
 

def average_AUC(y,f):

    """
    Task:    To compute average area under the ROC curves (AUC) given ten
             interaction threshold values from the pKd interval [6 M, 8 M]
             to binarize pKd's into true class labels.

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  avAUC   average AUC

    """

    thr = np.linspace(6,8,10)
    auc = np.empty(np.shape(thr)); auc[:] = np.nan

    for i in range(len(thr)):
        y_binary = copy.deepcopy(y)
        y_binary = preprocessing.binarize(y_binary.reshape(1,-1), threshold=thr[i], copy=False)[0]
        fpr, tpr, thresholds = metrics.roc_curve(y_binary, f, pos_label=1)
        auc[i] = metrics.auc(fpr, tpr)

    avAUC = np.mean(auc)

    return avAUC

def average_AUPRC(y,f):

    """
    Task:    To compute average area under the ROC curves (AUC) given ten
             interaction threshold values from the pKd interval [6 M, 8 M]
             to binarize pKd's into true class labels.

    Input:   y      Vector with original labels (pKd [M])
             f      Vector with predicted labels (pKd [M])

    Output:  avAUC   average AUC

    """

    thr = np.linspace(6,8,10)
    auc = np.empty(np.shape(thr)); auc[:] = np.nan

    for i in range(len(thr)):
        y_binary = copy.deepcopy(y)
        y_binary = preprocessing.binarize(y_binary.reshape(1,-1), threshold=thr[i], copy=False)[0]
        precision, recall, thresholds = metrics.precision_recall_curve(y_binary, f, pos_label=1)
        auc[i] = metrics.auc(recall, precision)

    avAUC = np.mean(auc)

    return avAUC



def get_list_of_scores():
    score_list = ["rm2", "CI", "MSE", "RMSE", "Pearson", "Spearman",
                  "Average AUC", "Average AUPRC",
                  "Precision 10uM", "Recall 10uM", "F1-Score 10uM", "Accuracy 10uM", "MCC 10uM",
                  "Precision 1uM", "Recall 1uM", "F1-Score 1uM", "Accuracy 1uM", "MCC 1uM",
                  "Precision 100nM", "Recall 100nM", "F1-Score 100nM", "Accuracy 100nM", "MCC 100nM",
                  "Precision 30nM", "Recall 30nM", "F1-Score 30nM", "Accuracy 30nM", "MCC 30nM",
                  ]
    return score_list


def get_validation_test_metric_list_of_scores():
    score_list =  get_list_of_scores()
    test_score_list = ["test {}".format(scr) for scr in score_list]
    validation_score_list = ["validation {}".format(scr) for scr in score_list]
    validation_test_metric_list = test_score_list + validation_score_list
    # print(validation_test_list)
    return validation_test_metric_list

def get_scores_generic(labels, predictions, validation_test, print_scores=False):

    score_dict = {"rm2": None, "CI": None, "RMSE": None, "MSE": None, "Pearson": None,
                  "Spearman": None, "Average AUC": None, "Average AUPRC":None,
                  "Precision 10uM": None, "Recall 10uM": None, "F1-Score 10uM": None, "Accuracy 10uM": None, "MCC 10uM": None,
                  "Precision 1uM": None, "Recall 1uM": None, "F1-Score 1uM": None, "Accuracy 1uM": None, "MCC 1uM": None,
                  "Precision 100nM": None, "Recall 100nM": None, "F1-Score 100nM": None, "Accuracy 100nM": None, "MCC 100nM": None,
                  "Precision 30nM": None, "Recall 30nM": None, "F1-Score 30nM": None, "Accuracy 30nM": None, "MCC 30nM": None,}
    score_list = get_list_of_scores()

    score_dict["rm2"] = get_rm2(np.asarray(labels), np.asarray(
        predictions))
    score_dict["CI"] = get_cindex(np.asarray(labels), np.asarray(
        predictions))
    score_dict["MSE"] = mse(np.asarray(labels), np.asarray(
        predictions))
    score_dict["RMSE"] = rmse(np.asarray(labels), np.asarray(
        predictions))
    score_dict["Pearson"] = pearson(np.asarray(labels), np.asarray(predictions))
    score_dict["Spearman"] = spearman(np.asarray(labels), np.asarray(predictions))
    score_dict["Average AUC"] = average_AUC(np.asarray(labels), np.asarray(predictions))
    score_dict["Average AUPRC"] = average_AUPRC(np.asarray(labels), np.asarray(predictions))

    prec_rec_f1_acc_mcc_threshold_dict = reg_prec_rec_f1_acc_mcc(np.asarray(labels), np.asarray(predictions))
    for key in prec_rec_f1_acc_mcc_threshold_dict.keys():
        score_dict[key] = prec_rec_f1_acc_mcc_threshold_dict[key]

    if print_scores:
        for scr in score_list:
            print("{} {}:\t{}".format(validation_test, scr, score_dict[scr]))
    return score_dict

def get_scores(labels, predictions, validation_test, total_training_loss, total_validation_test_loss, epoch, fold_epoch_results, print_scores=False, fold=None):

    score_dict = {"rm2": None, "CI": None, "MSE": None, "Pearson": None,
                  "Spearman": None, "Average AUC": None, "Average AUPRC":None,
                  "Precision 10uM": None, "Recall 10uM": None, "F1-Score 10uM": None, "Accuracy 10uM": None, "MCC 10uM": None,
                  "Precision 1uM": None, "Recall 1uM": None, "F1-Score 1uM": None, "Accuracy 1uM": None, "MCC 1uM": None,
                  "Precision 100nM": None, "Recall 100nM": None, "F1-Score 100nM": None, "Accuracy 100nM": None, "MCC 100nM": None,
                  "Precision 30nM": None, "Recall 30nM": None, "F1-Score 30nM": None, "Accuracy 30nM": None, "MCC 30nM": None,}
    score_list = get_list_of_scores()

    score_dict["rm2"] = get_rm2(np.asarray(labels), np.asarray(
        predictions))
    score_dict["CI"] = get_cindex(np.asarray(labels), np.asarray(
        predictions))
    score_dict["MSE"] = mse(np.asarray(labels), np.asarray(
        predictions))
    score_dict["RMSE"] = rmse(np.asarray(labels), np.asarray(
        predictions))
    score_dict["Pearson"] = pearson(np.asarray(labels), np.asarray(predictions))
    score_dict["Spearman"] = spearman(np.asarray(labels), np.asarray(predictions))
    score_dict["Average AUC"] = average_AUC(np.asarray(labels), np.asarray(predictions))
    score_dict["Average AUPRC"] = average_AUPRC(np.asarray(labels), np.asarray(predictions))

    prec_rec_f1_acc_mcc_threshold_dict = reg_prec_rec_f1_acc_mcc(np.asarray(labels), np.asarray(predictions))
    for key in prec_rec_f1_acc_mcc_threshold_dict.keys():
        score_dict[key] = prec_rec_f1_acc_mcc_threshold_dict[key]

    if print_scores:
        if fold!=None:
            fold_epoch_results[-1].append(score_dict)
            print("Fold:{}\tEpoch:{}\tTraining Loss:{}\t{} Loss:{}".format(fold + 1, epoch, total_training_loss,
                                                                           validation_test, total_validation_test_loss))
        else:
            fold_epoch_results.append(score_dict)
            print("Epoch:{}\tTraining Loss:{}\t{} Loss:{}".format(epoch, total_training_loss, validation_test,
                                                                  total_validation_test_loss))
        for scr in score_list:
            print("{} {}:\t{}".format(validation_test, scr, score_dict[scr]))
    return score_dict

def evaluate_score(model,X,y, preds = False, learning_method = 'classif'):
    
    """
    Parameters
    ----------
    model: model type used to predict new label
    X: feature matrix
    y: label matrix

    Returns
    -------
    Scores: recall, precision, f1, acc, f 0.5, mcc scores
    f: predicted label

    """
    f = model.predict(X)
    if learning_method == 'rgr':
        a = mse(y,f)
        b = rmse(y,f)
        c = spearman(y,f)
        d = pearson(y,f)
        e = average_AUC(y,f)
        g = reg_prec_rec_f1_acc_mcc(y,f)
        
        Scores = {'MSE':a,'RMSE':b,'Spearman':c,'Pearson':d,'Average_AUC':e,'threshold based Metrics':g}
        
    else:
        Scores = cl_prec_rec_f1_acc_mcc(y,f)
    
    if preds:
        return Scores,f
    return Scores


#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
from sklearn.cluster import KMeans

def kmeans(encodings, **kw):
	nclusters = int(kw['nclusters']) if kw['nclusters'] != None else 3
	if kw['sof'] == 'sample':
		encodings = np.array(encodings)[1:]
	else:
		encodings = np.array(encodings).T[1:]

	if len(encodings) < nclusters:
		return 0, 'sample number should be greater than n_clusters.'

	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)
	e = ''
	try:
		cluster_pred = KMeans(n_clusters = nclusters).fit_predict(data)
	except ValueError as e:
		print(e)
		return 0, e

	res = []
	for i in range(len(data)):
		res.append([encodings[i][0], cluster_pred[i]])
	return res, e





# -*- coding: utf-8 -*-
"""
Created on Tue Jan 11 18:26:00 2022

@author: Sameitos
"""

from profab.utils.imp_split_form import separator
import numpy as np
import random

def ttv_split(X = None,y = None, ratio = 0.2, X_pos = None, X_neg = None):
    
    '''
    Description:
        This function splits X and y randomly to train, test and validation
        sets according to ratio value
        
    Parameters:
        X: default = None, feature matrix. If X_pos and X_neg are not None,
            it has to stay None.
        y: default = None, label Matrix. If X_pos and X_neg are not None,
            it has to stay None. If X is defined, it cannot stay None.
        X_pos: default = None, positive set feature matrix. If X is defined,
                it has to stay None
        X_neg: default = None, negative set feature matrix. If X is defined,
                it has to stay None.
        ratio, type = {float,list}, (default = 0.2), is used to split the 
        data according given value(s). If ratio = a (flaot), then test will 
        be a% of total data size.If ratio = [a,b] where a and b are 
        in (0,1), train, test and validation sets are formed according to 
        them. If a = 0.2 and b = 0.1, train fraction is 0.7, test fraction 
        is 0.2 and validation fraction is 0.1 of all dataset size. 
    
    '''
    if X is not None and y is not None:
        return separator(X,y, ratio)

    elif X is not None and y is None:
        raise ValueError('While X is not None, y cannot be None.')

    elif X is None and X_pos is not None:
        if type(X_pos) == np.ndarray:
            X_pos = X_pos.tolist()
            X_neg = X_neg.tolist()
        y_pos = [1 for i in range(len(X_pos))]
        y_neg = [-1 for i in range(len(X_neg))]

        X = X_pos + X_neg
        y = y_pos + y_neg
        trdn = list(zip(X,y))
        random.shuffle(trdn)
        X,y = zip(*(trdn))

        return separator(X,y, ratio)

    elif X is None and X_neg is None and X_pos is None:
        raise ValueError('X, X_pos and X_neg data cannot be None at the same time.')
    


#!/usr/bin/env python
#_*_coding:utf-8_*_

import argparse
import numpy as np
import pandas as pd
import matplotlib as mpl
mpl.use('Agg')
import pylab
from clusters import *

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.",
									 description="cluster for the generated numerical represention")
	parser.add_argument("--file", required=True, help="input encoding file")
	parser.add_argument("--type", required=True,
						choices=['kmeans', 'hcluster', 'apc', 'meanshift', 'dbscan'], help="select cluster method")
	parser.add_argument("--sof", choices = ['sample', 'feature'], help="cluster for sample or feature, default: sample")
	parser.add_argument("--nclusters", help="specify the cluster number for kmeans cluster method. default: 3")
	parser.add_argument("--out", help="output file")
	args = parser.parse_args()

	kw = {'nclusters': args.nclusters if args.nclusters != None else 3, 'sof':args.sof if args.sof !=None else 'sample'}

	output = args.out if args.out != None else '%s_cluster.txt' %args.type
	encodings = readCode.readCode(args.file)
	myFun = args.type + '.' + args.type + '(encodings, **kw)'
	print('Cluster type: ' + args.type)
	myCluster, e = eval(myFun)
	saveCluster.saveCluster(myCluster, e, output)
	## t-sne plot

	if myCluster != 0:
		if kw['sof'] == 'sample':
			data = np.array(encodings)[1:, 1:].astype(float)
		else:
			data = np.array(encodings).T[1:, 1:].astype(float)
		labels = np.array(myCluster)[0:, 1:].reshape(-1, )
		e = ''
		try:
			Y = tsne.tsne(data, 2, 50, 20.0)
		except RuntimeWarning as e:
			Y = pca.pca(data, n_components = 2)

		df = pd.DataFrame({'X': Y[:, 0], 'Y': Y[:, 1], 'L': labels})

		mySet = set(labels)
		if len(mySet) > 5:
			pylab.scatter(Y[:, 0], Y[:, 1], 20, labels)
		else:
			for l in mySet:
				newData = df.loc[df.loc[:, "L"] == l, :]
				pylab.scatter(np.array(newData.X), np.array(newData.Y), 20, label="Cluster_%s" % l)
		pylab.legend(loc='best')
		#pylab.show()
		pylab.savefig('%s.png' %output)
		pylab.close()


#!/usr/bin/env python
#_*_coding:utf-8_*_

import re, os, sys

def readFasta(file):
	if os.path.exists(file) == False:
		print('Error: "' + file + '" does not exist.')
		sys.exit(1)

	with open(file) as f:
		records = f.read()

	if re.search('>', records) == None:
		print('The input file seems not in fasta format.')
		sys.exit(1)

	records = records.split('>')[1:]
	myFasta = []
	for fasta in records:
		array = fasta.split('\n')
		name, sequence = array[0].split()[0], re.sub('[^ARNDCQEGHILKMFPSTWYV-]', '-', ''.join(array[1:]).upper())
		myFasta.append([name, sequence])
	return myFasta


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta

def DisorderB(fastas, **kw):
	if checkFasta.checkFasta(fastas) == False:
		print('Error: for "DisorderB" encoding, the input fasta sequences should be with equal length. \n\n')
		return 0

	disDir = kw['path']
	if disDir == None:
		print('Error: please specify the directory of predicted protein disorder files by "--path"')
		return 0

	encodings = []
	header = ['#']
	for p in range(1, 2*len(fastas[0][1])+1):
		header.append('disorderB.F' + str(p))

	encodings.append(header)
	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		if os.path.exists(disDir + '/' + name + '.dis') == False:
			print('Error: the predicted disorder information file (.dis) for protein ' + name + ' does not exist.')
			return 0
		with open(disDir + '/' + name + '.dis') as f:
			records = f.readlines()
		tag = 0
		for i in range(len(records)):
			if re.search('^-------', records[i]):
				tag = i
				break
		records = records[tag+1:-1]

		proteinSeq = ''
		disValue = []
		myDict = {'D':[0, 1], 'O':[1, 0]}
		for line in records:
			array = line.rstrip().split() if line.rstrip() != '' else None
			key = array[3] if array[3] == 'D' else 'O'
			proteinSeq = proteinSeq + array[1]
			disValue.append(key)

		pos = proteinSeq.find(sequence)
		if pos == -1:
			print('Warning: could not find the peptide in proteins.\n\n')
		else:
			for p in range(pos, pos+len(sequence)):
				code = code + myDict[disValue[p]]
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

__all__ = ['readFasta','saveCode', 'checkFasta',
		   'AAC', 'EAAC', 'CKSAAP', 'DPC', 'TPC', 'DDE', 'BINARY', 'BLOSUM62', 'AAINDEX', 'ZSCALE',
		   'GAAC', 'EGAAC', 'CKSAAGP', 'GDPC', 'GTPC',
		   'NMBroto', 'Moran', 'Geary',
		   'CTDC', 'CTDT', 'CTDD',
		   'CTriad', 'KSCTriad',
		   'SOCNumber', 'QSOrder',
		   'PAAC', 'APAAC',
		   'KNNprotein', 'KNNpeptide',
		   'PSSM', 'SSEC', 'SSEB', 'Disorder', 'DisorderC', 'DisorderB', 'ASA', 'TA',
		   'NeedlemanWunsch',
		   ]


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
import shutil
pPath = re.sub(r'scripts$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
from codes import readFasta
import argparse


def generateSecondaryStructure(fastas, outDir, psipred):
	"""
	Generate predicted protein secondary structure by using the 'PSIPRED Version 4.01' program.

	Parameters
	----------
	file : file
		the file, which include the protein sequences in fasta format.

	psipred: string
		the path of PSIPRED Version 4.01 program, the PSIPRED Version 4.01 program could be download at: http://bioinfadmin.cs.ucl.ac.uk/downloads/psipred/.

	Returns
	-------
	a string: 
		A directory name, which include the predicted protein secondary structure information.
	"""

	if os.path.exists(outDir) == False:
		os.mkdir(outDir)

	for i in fastas:
		name, sequence = re.sub('\|', '', i[0]), i[1]
		with open(name + '.txt', 'w') as f:
			f.write('>'+name+'\n'+sequence + '\n')
		myCmd = psipred + ' ' + name + '.txt'
		if os.path.exists(outDir + '/' + name + '.ss2') == False:
			os.system(myCmd)
			os.remove(name + '.txt')
			os.remove(name + '.ss')
			os.remove(name + '.horiz')
			shutil.move(name+'.ss2', outDir)
	return outDir

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.", description="generate protein secondary structure profile")
	parser.add_argument("--file", required=True, help="protein sequence file in fasta format")
	parser.add_argument("--psipred", help="the path of psipred program")
	args = parser.parse_args()

	psipred = args.psipred if args.psipred != None else 'runpsipred'
	fastas = readFasta.readFasta(args.file)
	outputDir = generateSecondaryStructure(fastas, 'out', psipred)
	print('The predicted secodnary structure are stored in: ' + outputDir)


# -*- coding: utf-8 -*-
"""
Created on Tue Jan 11 18:25:15 2022

@author: Sameitos
"""

from profab.utils.feature_extraction_module.feature_extracter import feature_extracter
from profab.utils.feature_extraction_module.utils import bcolors

def extract_protein_feature(protein_feature,
                          place_protein_id,
                          input_folder,
                          fasta_file_name
                          ):


    feat_ext = feature_extracter(protein_feature,
                                 place_protein_id,
                                 input_folder,
                                 fasta_file_name)

    if protein_feature in feat_ext.POSSUM_desc_list:

        return feat_ext.extract_POSSUM_feature()

    elif protein_feature in feat_ext.iFeature_desc_list:

        return feat_ext.extract_iFeature_feature()

    else:

        print(f"{bcolors.FAIL}Protein Feature extraction method is not in either POSSUM or iFeature{bcolors.ENDC}")



#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re

def calculateDicorderContent(pos, endPos, disValue):
	newValues = disValue[pos: endPos]
	return [newValues.count('D')/(endPos - pos), newValues.count('O')/(endPos-pos)]

def DisorderC(fastas, **kw):
	encodings = []
	header = ['#', 'disorder-content', 'order-content']
	encodings.append(header)

	disDir = kw['path']
	if disDir == None:
		print('Error: please specify the directory of predicted protein disorder files by "--path"')
		return 0

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		if os.path.exists(disDir + '/' + name + '.dis') == False:
			print('Error: the predicted disorder information file (.dis) for protein ' + name + ' does not exist.')
			return 0

		with open(disDir + '/' + name + '.dis') as f:
			records = f.readlines()
		tag = 0
		for i in range(len(records)):
			if re.search('^-------', records[i]):
				tag = i
				break
		records = records[tag+1:-1]

		proteinSeq = ''
		disValue = []
		for line in records:
			array = line.rstrip().split() if line.rstrip() != '' else None
			key = array[3] if array[3] == 'D' else 'O'
			proteinSeq = proteinSeq + array[1]
			disValue.append(key)

		pos = proteinSeq.find(sequence)
		if pos == -1:
			print('Warning: could not find the peptide in proteins.\n\n')
		else:
			code = code + calculateDicorderContent(pos, pos+len(sequence), disValue)
		encodings.append(code)

	return encodings


#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
import math
import numpy as np
pPath = re.sub(r'codes$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
from codes import readFasta
from Bio import pairwise2
from Bio.SubsMat.MatrixInfo import blosum62

def CalculateSimilarity(sequence1, sequence2):
	alignment = pairwise2.align.globalds(sequence1, sequence2, blosum62, -10, -0.5)[0]
	sum = 0
	for i in range(len(alignment[0])):
		if alignment[0][i] == alignment[1][i]:
			sum = sum + 1
	return 2 * sum / (len(sequence1) + len(sequence2))

def CalculateContent(mySimilarity, j, myLabelSets):
	content = []
	myDict = {}
	for i in myLabelSets:
		myDict[i] = 0
	for i in range(j):
		myDict[mySimilarity[i][0]] = myDict[mySimilarity[i][0]] + 1
	for i in myLabelSets:
		content.append(myDict[myLabelSets[i]] / j)
	return content

def KNNprotein(fastas, **kw):
	trainFile = kw['train']
	labelFile = kw['label']

	if os.path.exists(labelFile) == False:
		print('Error: the label file does not exist.')
		sys.exit(1)

	if trainFile == None or labelFile == None:
		print('Error: please specify the directory of train file ["--train"] and the label file ["--label"]')
		sys.exit(1)
	trainData = readFasta.readFasta(trainFile)
	with open(labelFile) as f:
		records = f.readlines()
	myLabel = {}
	for i in records:
		array = i.rstrip().split() if i.strip() != '' else None
		myLabel[array[0]] = int(array[1])
	myLabelSets = list(set(myLabel.values()))

	if len(trainData) != len(myLabel):
		print('ERROR: the inconsistent sample number in train and label file.')
		sys.exit(1)

	kValues = [0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.10, 0.11, 0.12, 0.13, 0.14, 0.15,
			   0.16, 0.17, 0.18, 0.19, 0.20, 0.21, 0.22, 0.23, 0.24, 0.25, 0.26, 0.27, 0.28, 0.29, 0.30]
	kNum = []
	for i in kValues:
		kNum.append(math.ceil(len(trainData) * i))

	encodings = []
	header = ['#']
	for k in kValues:
		for l in myLabelSets:
			header.append('Top' + str(k) + '.label' + str(l))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('[^ARNDCQEGHILKMFPSTWYV-]', '', i[1])
		code = [name]
		mySimilarity = []
		for j in range(len(trainData)):
			if name != trainData[j][0]:
				mySimilarity.append([myLabel[trainData[j][0]], CalculateSimilarity(re.sub('[^ARNDCQEGHILKMFPSTWYV]', '', trainData[j][1]), sequence)])
		mySimilarity = np.array(mySimilarity)
		mySimilarity = mySimilarity[np.lexsort(-mySimilarity.T)]
		for j in kNum:
			code = code + CalculateContent(mySimilarity, j, myLabelSets)
		encodings.append(code)
	return encodings




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Authors: Chris (chris@nohup.cc) & Young (young@nohup.cc)

def usage():
    print("possum.py usage:")
    print("python possum.py <options> <source files> ")
    print("-i,--input: input a file in fasta format.")
    print("-o,--ouput: output a file of the generated feature.")
    print("-t,--type: specify a feature encoding algorithm.")
    print("-p,--pssmdir: specify the directory of pssm files.")
    print("-h,--help: show the help information.")

import fileinput
import sys, getopt
from os import listdir
from os.path import isfile, join
import re
import numpy as np
from possum_ft import *

opts, args = getopt.getopt(sys.argv[1:], 'i:o:t:p:a:b:h', ['input=','output=','type=','pssmdir=','argument=','veriable=','help'])
inputFile=""
outputFile=""
algoType=""
pssmdir=""
argument=""
veriable=""

for opt, arg in opts:
    if opt in ('-i','--input'):
        inputFile = arg
    elif opt in ('-o','--output'):
        outputFile = arg
    elif opt in ('-t','--type'):
        algoType = arg
    elif opt in ('-p','--pssmdir'):
        pssmdir = arg
    elif opt in ('-a','--argument'):
        argument = int(arg)
    elif opt in ('-b','--veriable'):
        veriable = int(arg)
    elif opt in ('-h', '--help'):
        usage()
        sys.exit(2)
    else:
        usage()
        sys.exit(2)
check_head = re.compile(r'\>')

smplist = []
smpcnt = 0
seq = ""

for line, strin in enumerate(fileinput.input(inputFile)):
    if not check_head.match(strin):
        seq += strin.strip()
        #smplist.append(strin.strip())
        #smpcnt += 1
    elif seq!="":
        smplist.append(seq)
        smpcnt += 1
        seq= ""
smplist.append(seq)
smpcnt += 1

#print "smplist="
#print smplist
onlyfiles = [ f for f in listdir(pssmdir) if isfile(join(pssmdir,f)) ]
#print "onlyfiles="
#print onlyfiles
fastaDict = {}

for fi in onlyfiles:
    cntnt = ''
    pssmContentMatrix=readToMatrix(fileinput.input(pssmdir+'/'+fi))
    #print(pssmContentMatrix)
    pssmContentMatrix=np.array(pssmContentMatrix)
    #print(pssmContentMatrix)
    sequence=pssmContentMatrix[:,0]
    seqLength=len(sequence)
    for i in range(seqLength):
        cntnt+=sequence[i]
    if cntnt in fastaDict:
        #print strin
        continue
    fastaDict[cntnt] = fi

finalist = []
for smp in smplist: 
    #print "smp="+smp
    #print "fastaDict[smp]="+fastaDict[smp]
    finalist.append(pssmdir+'/'+fastaDict[smp])

file_out = open(outputFile,'w')

for fi in finalist:
    #print "fi=" + fi
    #pass in original matrix
    input_matrix=fileinput.input(fi)
    #output a feature vector
    #input_matrix=readToMatrix(input_matrix)
    #print(input_matrix)
    feature_vector=calculateDescriptors(input_matrix, algoType, argument, veriable)
    np.savetxt(file_out, feature_vector, delimiter=",")



#!/usr/bin/env python
#_*_coding:utf-8_*_

USAGE = """
USAGE:
	python KSCTriad.py input.fasta <K> output
	
	input.fasta:  the input protein sequence file in fasta format.
	K:            the max space number, integer, defaule: 5
	output:       the encoding file, default: 'encodings.tsv'
"""
import re, sys, os
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import readFasta
import saveCode

def CalculateKSCTriad(sequence, gap, features, AADict):
	res = []
	for g in range(gap+1):
		myDict = {}
		for f in features:
			myDict[f] = 0

		for i in range(len(sequence)):
			if i+g+1 < len(sequence) and i+2*g+2<len(sequence):
				fea = AADict[sequence[i]] + '.' + AADict[sequence[i+g+1]]+'.'+AADict[sequence[i+2*g+2]]
				myDict[fea] = myDict[fea] + 1

		maxValue, minValue = max(myDict.values()), min(myDict.values())
		for f in features:
			res.append((myDict[f] - minValue) / maxValue)

	return res

def KSCTriad(fastas, gap = 0, **kw):
	AAGroup = {
		'g1': 'AGV',
		'g2': 'ILFP',
		'g3': 'YMTS',
		'g4': 'HNQW',
		'g5': 'RK',
		'g6': 'DE',
		'g7': 'C'
	}

	myGroups = sorted(AAGroup.keys())

	AADict = {}
	for g in myGroups:
		for aa in AAGroup[g]:
			AADict[aa] = g

	features = [f1 + '.'+ f2 + '.' + f3 for f1 in myGroups for f2 in myGroups for f3 in myGroups]

	encodings = []
	header = ['#']
	for g in range(gap+1):
		for f in features:
			header.append(f+'.gap'+str(g))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], re.sub('-', '', i[1])
		code = [name]
		if len(sequence) < 2*gap + 3:
			print('Error: for "KSCTriad" encoding, the input fasta sequences should be greater than (2*gap+3). \n\n')
			return 0
		code = code + CalculateKSCTriad(sequence, gap, features, AADict)
		encodings.append(code)

	return encodings

if __name__ == '__main__':
	if len(sys.argv) == 1:
		print(USAGE)
		sys.exit(1)
	fastas = readFasta.readFasta(sys.argv[1])
	k = int(sys.argv[2]) if len(sys.argv) >= 3 else 5
	output = sys.argv[3] if len(sys.argv) >= 4 else 'encoding.tsv'
	encodings = KSCTriad(fastas, k)
	saveCode.savetsv(encodings, output)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Authors: Chris (chris@nohup.cc) & Young (young@nohup.cc)

def usage():
    print("headerHandler.py usage:")
    print("python headerHandler.py <options> <source files> ")
    print("-i,--input: input raw feature file without header.")
    print("-o,--ouput: output feature file with header.")
    print("-p,--prefix: input the prefix of header.")
    print("-n,--number: input the number of columns")
    print("-h,--help: show the help information.")

import re
import pandas as pd
import numpy as np
import fileinput
import sys, getopt
from os import listdir
from os.path import isfile, join

opts, args = getopt.getopt(sys.argv[1:], 'i:o:p:n:h', ['input=','output=','prefix=','number=','help'])
inputFile=""
outputFile=""
prefix=""
colNumber=""

for opt, arg in opts:
    if opt in ('-i','--input'):
        inputFile = arg
    elif opt in ('-o','--output'):
        outputFile = arg
    elif opt in ('-p','--prefix'):
        prefix = arg
    elif opt in ('-n','--number'):
        colNumber = int(arg)
    elif opt in ('-h', '--help'):
        usage()
        sys.exit(2)
    else:
        usage()
        sys.exit(2)

df_input = pd.read_csv(inputFile, header = None)

headerRow=[]
for i in range(colNumber):
    headerRow.append(prefix+str(i))
df_input.columns=headerRow

df_input.to_csv(outputFile,index=False)



#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os, re
import pandas as pd
pPath = re.sub(r'scripts$', '', os.path.split(os.path.realpath(__file__))[0])
sys.path.append(pPath)
from clusters import readCode
import argparse
import numpy as np
from sklearn.decomposition import PCA
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

def pacAnalysis(encodings, n_components = 2):
	encodings = np.array(encodings)[1:]
	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)
	newData = PCA(n_components = n_components).fit_transform(data)
	pca = []
	for i in range(len(data)):
		pca.append([encodings[i][0]] + list(newData[i]))
	return pca

def savePCA(pca, file='PCA.txt'):
	with open(file, 'w') as f:
		f.write('protein\t')
		for i in range(1, len(pca[0])):
			f.write('pc.' + str(i) + '\t')
		f.write('\n')
		for i in pca:
			for j in i:
				f.write(str(j) + '\t')
			f.write('\n')
	return None

def pcaPlot(pca, label, file='pca.png'):
	pca = np.array(pca)[:]
	data = pca[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	if len(label) == 0:
		plt.scatter(data[:, 0], data[:, 1], 20, c='r')
	else:
		df = pd.DataFrame({'X': data[:, 0], 'Y': data[:, 1], 'L': label})
		mySet = set(label)
		for l in mySet:
			newData = df.loc[df.loc[:, "L"] == l, :]
			plt.scatter(np.array(newData.X), np.array(newData.Y), 20, label="%s" % l)
			plt.legend(loc='best')
	plt.savefig(file)
	return None

if __name__ == '__main__':
	parser = argparse.ArgumentParser(usage="it's usage tip.", description="PCA and plot")
	parser.add_argument("--file", required=True, help="data file for PCA")
	parser.add_argument("--out", help="output file of PCA analysis.")
	parser.add_argument("--ncomponents", help="number of n components, default 2")
	parser.add_argument("--label", help="optimal, the label file for the input file")
	args = parser.parse_args()
	output = args.out if args.out != None else 'PCA.txt'
	n_components = int(args.ncomponents) if args.ncomponents != None else 2
	data = readCode.readCode(args.file)
	label = []
	if args.label != None:
		with open(args.label) as f:
			records = f.readlines()
		for line in records:
			array = line.strip().split() if line.strip() != '' else None
			label.append(array[1])

	pca = pacAnalysis(data, n_components)
	savePCA(pca, output)
	pcaPlot(pca, label, '%s.png' %output)






#!/usr/bin/env python
#_*_coding:utf-8_*_

import sys, os
pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
import checkFasta

def BINARY(fastas, **kw):
	if checkFasta.checkFasta(fastas) == False:
		print('Error: for "BINARY" encoding, the input fasta sequences should be with equal length. \n\n')
		return 0

	AA = 'ARNDCQEGHILKMFPSTWYV'
	encodings = []
	header = ['#']
	for i in range(1, len(fastas[0][1]) * 20 + 1):
		header.append('BINARY.F'+str(i))
	encodings.append(header)

	for i in fastas:
		name, sequence = i[0], i[1]
		code = [name]
		for aa in sequence:
			if aa == '-':
				code = code + [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
				continue
			for aa1 in AA:
				tag = 1 if aa == aa1 else 0
				code.append(tag)
		encodings.append(code)
	return encodings



#!/usr/bin/env python
#_*_coding:utf-8_*_

import numpy as np
from sklearn.decomposition import PCA

def pca(encodings, n_components = 2):
	encodings = np.array(encodings)
	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)
	newData = PCA(n_components = n_components).fit_transform(data)
	pca = []
	for i in range(len(data)):
		pca.append([encodings[i][0]] + list(newData[i]))
	return np.array(pca)


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description_ = fh.read()
requirements = ['wget==3.2', 'tqdm==4.63.0','requests==2.27.1', 'numpy==1.21.2', 'scikit-learn==1.0.1']
setuptools.setup(
    name='profab-pypi',
    version='1.0',
    author='samet ozdilek',
    author_email='oz.samet.473@gmail.com',
    description='Installation of Package',
    long_description= long_description_,
    long_description_content_type="text/markdown",
    url='https://github.com/Sametle06/ProFAB',
    project_urls = {
        "Bug Tracker": "https://github.com/Sametle06/ProFAB/issues"
    },
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
)



# -*- coding: utf-8 -*-
"""
Created on Tue Mar  9 01:00:05 2021

@author: Sameitos
"""

import os, re
import random
from zipfile import ZipFile
from profab.utils.imp_split_form import separator,  self_data, _classif_data_import, download_data

class cls_data_loader():
    
    def __init__(self,ratio,protein_feature,main_set,set_type,label,pre_determined):
        
        """
            Parameters:
                main_set: {ec_dataset, go_dataset}, folder includes its corresponding data name
        """
        self.ratio = ratio
        self.protein_feature = protein_feature
        self.set_type = set_type
        self.label = label
        self.pre_determined = pre_determined
        self.main_set = main_set#{ec_dataset,go_dataset}: Indicated which data folder will be opened.
        self.server_path = "https://liverdb.kansil.org/profab"
        self.raiser()
        
        
    def raiser(self):
        
        """
            It is to raise the exceptions that can be occured because of false attributes enter.
        """
        if self.set_type not in ['random','similarity','temporal']:
         	raise AttributeError('Please enter correct set_type. Options are: "random, similarity, temporal"')
        if self.set_type == 'similarity': self.set_type = 'target'
        if self.set_type == 'temporal': 
            self.pre_determined = True
        
        if self.protein_feature not in ['paac', 'aac', 'gaac', 'ctdt','ctriad','socnumber', 'kpssm']:
         	raise AttributeError('Please enter correct protein_feature. Options are: "paac, aac, gaac, ctdt, ctriad, socnumber, kpssm"')
        if self.protein_feature == 'kpssm': self.protein_feature = 'k_separated_bigrams_pssm'
        
        
        if self.label not in [None,'positive','negative','positive_negative']:
            raise AttributeError('Please enter correct label. Options are: "None, positive, negative"')
        
        
        if self.pre_determined:
            if type(self.ratio) not in [None,float]:
                raise AttributeError(
                    f'Please enter ratio value in true type. Options: "None, float" for pre_determined = True or '
                    f'set_type = temporal')        
        elif not self.pre_determined:
            if type(self.ratio) not in [None,float,list]:
                raise AttributeError(
                    'Please enter ratio value in true type. Options: "None, float and list" for pre_determined = False')
        
        
    def get_data(self,data_name):
        
        """
            Take attributes and prepare datasets according to them. If no data is available at local, then is dowloaded
            
        """
        
        #Check whether given data name and its family are matched
        if self.main_set[:2].lower() != data_name[:2].lower():
            
            raise FileNotFoundError(f'Please enter a correct data name: {data_name} not found in {self.main_set[:2].upper()} sets')
        
        pPath = os.path.split(os.path.realpath(__file__))[0]
        
        #Check whether there is save folder
        if not os.path.exists(self.main_set):
            os.makedirs(self.main_set)
        #Check whether there is a folder named main set
        if os.path.isfile(self.main_set + '/' + data_name + '.zip'):
            data_path = self.main_set + '/' + data_name + '.zip'

            
        #If no given data name, then data will be downloaded from the server to folder of main set.
        #This condition also looks for if data is available in server.
        else:
            print(f'No local dataset for {data_name} is available. Downloading from server ...')
            
            data_server_path = self.server_path + '/' + self.main_set + '/' + data_name + '.zip'
            
            data_path = self.main_set + '/' + data_name + '.zip'
            
            download_data(data_server_path,data_path)
            
        #Rest is checking wheter files are optional and preparing datasets
        if not self.pre_determined:

            pos_file = data_name + '/' + self.set_type + '_positive_' + self.protein_feature + '.txt'
            neg_file = data_name + '/' + self.set_type + '_negative_' + self.protein_feature + '.txt'
            
            if pos_file not in ZipFile(data_path).namelist() or neg_file not in ZipFile(data_path).namelist():
                self.look_options(ZipFile(data_path).namelist(), data_name)

            pX,nX,X,y = _classif_data_import(zip_data = data_path, pos_file = 
                                               pos_file, neg_file = neg_file, label = self.label)


            if self.label == 'positive':
                return pX
            elif self.label == 'negative':
                return nX
            
            else:
                trdn = list(zip(X,y))
                random.shuffle(trdn)
                X,y = zip(*(trdn))
                
                if not self.ratio:
                    return X,y
                if self.ratio is not None:
                    return separator(ratio = self.ratio,X = X,y =y)
                else:
                    raise AttributeError(
                        'Please enter ratio value in true type. Options: "None, float and list" for pre_determined = False')
                    
        else:
        
            pos_file = data_name + '/' + self.set_type + '_positive_' + self.protein_feature + '.txt'
            neg_file = data_name + '/' + self.set_type + '_negative_' + self.protein_feature + '.txt'
            
            if pos_file not in ZipFile(data_path).namelist() or neg_file not in ZipFile(data_path).namelist():
                
                self.look_options(ZipFile(data_path).namelist(), data_name)
                
            
            
            train_pos_idx = data_name + '/' + self.set_type + '_positive_train_indices.txt'
            train_neg_idx = data_name + '/' + self.set_type + '_negative_train_indices.txt'

            test_pos_idx = data_name + '/' + self.set_type + '_positive_test_indices.txt'
            test_neg_idx = data_name + '/' + self.set_type + '_negative_test_indices.txt'
                        

            tpX,tnX,tX,ty = _classif_data_import(zip_data = data_path, pos_file = pos_file,
                                                         neg_file = neg_file, pos_indices = train_pos_idx,
                                                         neg_indices = train_neg_idx,
                                                         label = self.label)
            tepX,tenX,teX,tey = _classif_data_import(zip_data = data_path, pos_file = pos_file,
                                                               neg_file = neg_file,
                                                               pos_indices = test_pos_idx,
                                                               neg_indices = test_neg_idx,
                                                               label = self.label)
                

            return_pos = [tpX,tepX]
            return_neg = [tnX,tenX]
            
            if self.set_type == 'temporal':
                valid_pos_idx = data_name + '/' + self.set_type + '_positive_validation_indices.txt'
                valid_neg_idx = data_name + '/' + self.set_type + '_negative_validation_indices.txt'
                
                vpX,vnX,vX,vy = _classif_data_import(zip_data = data_path,pos_file = pos_file, neg_file = neg_file, 
                    pos_indices = valid_pos_idx,neg_indices = valid_neg_idx,
                    label = self.label)            
                
                return_pos.extend([vpX])
                return_neg.extend([vnX])

            
            if self.label == 'positive':
                return return_pos
            elif self.label == 'negative':
                return return_neg
            elif self.label == 'positive_negative':
                return return_pos.extend(return_neg)
            else:
                
                trdn = list(zip(tX,ty))
                random.shuffle(trdn)
                tX,ty = zip(*trdn)
            
                terdn = list(zip(teX,tey))
                random.shuffle(terdn) 
                teX,tey = zip(*terdn)
            
                if self.set_type == 'temporal':
                    vrdn = list(zip(vX,vy))
                    random.shuffle(vrdn)
                    vX,vy = zip(*vrdn)
                    return tX,teX,vX,ty,tey,vy
            
            
                if self.ratio is None:
                    return tX,teX,ty,tey
                    
                if type(self.ratio) == float:
                    tX,vX,ty,vy = separator(ratio = self.ratio,X = tX,y = ty)
                    return tX,teX,vX,ty,tey,vy
                
                else:
                    raise AttributeError(
                        'Please enter ratio value in true type. Options: "None, float" for pre_determined = True')
                    
    def look_options(self,name_list,data_name):
        """
            Look for options for set_type and protein features found 
            in zip file and return them. It is to learn what is inside
            of zip file.
        """
        avai_sets, avai_prots = set(),set()
        for names in name_list:
            if not re.search('indices',names):
                row = re.split('_|/',names)
                avai_sets.add(row[2])
                avai_prots.add(row[-1][:-4])
        if 'target' in avai_sets:
            avai_sets.remove('target')
            avai_sets.add('similarity')
        if 'k_separated_bigrams_pssm' in avai_prots:
            avai_prots.remove('k_separated_bigrams_pssm')
            avai_prots.add('kpssm')
        
        if self.protein_feature == 'k_separated_bigrams_pssm': self.protein_feature = 'kpssm'
        if self.set_type == 'target': self.set_type = 'similarity'
        if self.set_type not in avai_sets:
            raise Exception(
                
                f'!!Under maintenance!! Specified set type "{self.set_type}" is not available for {data_name}. '
                f'Available set_type options: {avai_sets}')
        elif self.protein_feature not in avai_prots:
            raise Exception(
                
                f'!!Under maintenance!! Specified protein feature type '
                f'"{self.protein_feature}" type is not available for {data_name}. '
                f'Available protein_feature options: {avai_prots}')
        
        else:
            raise Exception(
                
                f'!!Under maintenance!! Specified set type "{self.set_type}" and protein feature type '
                f'"{self.protein_feature}" are not available for {data_name}. '
                f'Available set_type options: '
                f'{avai_sets} and available protein_feature options: {avai_prots}')
        
        
        
        return avai_sets, avai_prots


class casual_importer():
    
    def __init__(self,delimiter, name, label):
        
        self.delimiter = delimiter
        self.name = name
        self.label = label
        
    def get_data(self,file_name):
        
        """
        Parameters:
            file_name = Name of file holds data. 
                        Format must be specified.
        """
        
        return self_data(file_name = file_name,
                         delimiter = self.delimiter,
                         name = self.name,
                         label = self.label)
    



# -*- coding: utf-8 -*-
"""
Created on Mon January 3 00:03:05 2022
@author: Gokhan Ozsari
"""

import re
from profab.utils.feature_extraction_module.utils import *
import warnings
import pathlib

warnings.filterwarnings('ignore')
path_to_folder = pathlib.Path(__file__).parent.resolve()

def usage():
    print(f"{bcolors.OKGREEN}")
    print("feature_extracter.py usage:\n")
    print("from profab.feature_extraction_module import *:\n\n\t\t it is to import the feature extraction module\n\n")
    print("feature_extracter.extract_protein_feature(protein_feature, place_protein_id, input_folder, fasta_file_name):\n\n\
          \t This function extracts the features of the fasta file (fasta_file_name) in the input folder (input_folder)\n\
          \t by using the protein feature extraction method (protein_feature)\n\n")
    print("Parameters:\n\nprotein_feature: {string}, (default = 'aac_pssm'):\n\n\t\t one of the 21 PSMM-based protein descriptors in POSSUM.\n\n\
          \t ac_pssm, d_fpssm, smoothed_pssm, ab_pssm, pssm_composition, rpm_pssm,\n\
          \t s_fpssm, dpc_pssm, k_separated_bigrams_pssm, eedp, tpc, edp, rpssm,\n\
          \t pse_pssm, dp_pssm, pssm_ac, pssm_cc, aadp_pssm, aatp, medp , or all_POSSUM\n\n\
          \t all_POSSUM: it extracts the features of all (21) POSSUM protein descriptors\n\n\
          or\n\n\
          one of the 18 protein descriptors in iFeature.\n\n\
          \t AAC, PAAC, APAAC, DPC, GAAC, CKSAAP, CKSAAGP, GDPC, Moran, Geary,\n\
          \t NMBroto, CTDC, CTDD, CTDT, CTriad, KSCTriad, SOCNumber, QSOrder, or all_iFeature\n\n\
          \t all_iFeature: it extracts the features of all (18) iFeature protein descriptors\n\n")
    print("place_protein_id: {int}, (default = 1):\n\n\
          \t It indicates the place of protein id in fasta header.\n\
          \t e.g. fasta header: >sp|O27002|....|....|...., seperate the header wrt. '|' then >sp is\n\
          \t in the zeroth position, protein id in the first(1) position.\n\n")
    print("input_folder: {string}, (default = 'input_folder'}:\n\n\
          \t it is the path to the folder that contains the fasta file.\n\n")
    print("fasta_file_name: {string}, (default ='sample'):\n\n\
          \t it is the name of the fasta file exclude the '.fasta' extension.\n")
    print(f"{bcolors.ENDC}")

class feature_extracter(object):
    '''
     The feature_extracter class is designed to extract features by employing POSSUM and iFeature python-based tools.

     POSSUM (Position-Specific Scoring matrix-based feature generator for machine learning),
     a versatile toolkit with an online web server that can generate 21 types of PSSM-based feature descriptors,
     thereby addressing a crucial need for bioinformaticians and computational biologists.

     iFeature, a versatile Python-based toolkit for generating various numerical feature representation schemes for
     both protein and peptide sequences. iFeature is capable of calculating and extracting a comprehensive spectrum
     of 18 major sequence encoding schemes that encompass 53 different types of feature descriptors.

     Parameters:
         protein_feature: {string}, (default = 'aac_pssm'): one of the 21 PSMM-based protein descriptors in POSSUM.

                          aac_pssm, d_fpssm, smoothed_pssm, ab_pssm, pssm_composition, rpm_pssm,
                          s_fpssm, dpc_pssm, k_separated_bigrams_pssm, eedp, tpc, edp, rpssm,
                          pse_pssm, dp_pssm, pssm_ac, pssm_cc, aadp_pssm, aatp, medp , or all_POSSUM

                          all_POSSUM: it extracts the features of all (21) POSSUM protein descriptors

                          or

                          one of the 18 protein descriptors in iFeature.

                          AAC, PAAC, APAAC, DPC, GAAC, CKSAAP, CKSAAGP, GDPC, Moran, Geary,
                          NMBroto, CTDC, CTDD, CTDT, CTriad, KSCTriad, SOCNumber, QSOrder, or all_iFeature

                          all_iFeature: it extracts the features of all (18) iFeature protein descriptors

         place_protein_id: {int}, (default = 1): It indicates the place of protein id in fasta header.
                           e.g. fasta header: >sp|O27002|....|....|...., seperate the header wrt. '|' then >sp is
                           in the zeroth position, protein id in the first(1) position.

        input_folder: {string}, (default = 'input_folder'}: it is the path to the folder that contains the fasta file.

        fasta_file_name: {string}, (default ='sample'): it is the name of the fasta file exclude the '.fasta' extension.
    '''

    def __init__(self, protein_feature='aac_pssm',
                 place_protein_id=1,
                 input_folder='input_folder',
                 fasta_file_name='sample'):

        self.protein_feature = protein_feature
        self.place_protein_id = place_protein_id
        self.input_folder = input_folder
        self.fasta_file_name = fasta_file_name

        self.POSSUM_desc_list = {'aac_pssm', 'd_fpssm', 'smoothed_pssm', 'ab_pssm', 'pssm_composition',
                                  'rpm_pssm', 's_fpssm', 'dpc_pssm', 'k_separated_bigrams_pssm', 'eedp',
                                  'tpc', 'edp', 'rpssm', 'pse_pssm', 'dp_pssm', 'pssm_ac', 'pssm_cc',
                                  'aadp_pssm', 'aatp', 'medp', 'tri_gram_pssm', 'all_POSSUM'}

        self.iFeature_desc_list = {'AAC', 'PAAC', 'APAAC', 'DPC', 'GAAC', 'CKSAAP', 'CKSAAGP', 'GDPC',
                                   'Moran', 'Geary', 'NMBroto', 'CTDC', 'CTDD', 'CTDT', 'CTriad',
                                   'KSCTriad', 'SOCNumber', 'QSOrder', 'all_iFeature'}
        temp_folder = '{}/temp_folder'.format(path_to_folder)
        temp_output_folder = '{}/temp_folder/temp_output_folder'.format(path_to_folder)
        pssm_dir = '{}/pssm_files'.format(path_to_folder)
        if os.path.isdir(temp_folder) == False:
            os.mkdir(temp_folder)
        if os.path.isdir(temp_output_folder) == False:
            os.mkdir(temp_output_folder)
        if os.path.isdir(pssm_dir) == False:
            os.mkdir(pssm_dir)
        ncbi = '{}/ncbi-blast'.format(path_to_folder)
        bool_var = False
        if os.path.isdir(ncbi) == False:
            download_extract_ncbi_blast(bool_var)
        else:
            bool_var = True
            download_extract_ncbi_blast(bool_var)


    def extract_POSSUM_feature(self):
        fasta_dict = read_fasta_to_dict(self.input_folder, self.fasta_file_name, self.place_protein_id)
        copy_form_pssm_matrices(fasta_dict)

        list_desc = [self.protein_feature]

        if self.protein_feature == 'all_POSSUM':

            list_desc = self.POSSUM_desc_list - {'all_POSSUM'}

        for prot_feat in list_desc:

            temp_output_file = '{}/temp_folder/temp_output_folder/{}_{}.txt'.format(path_to_folder,
                                                                                    self.fasta_file_name,
                                                                                    prot_feat)

            os.system('python {}/POSSUM_Standalone_Toolkit/src/possum.py -i {}/{}.fasta'\
                      ' -o {} -t {} -p {}/pssm_files'.format(path_to_folder,
                                                          self.input_folder,
                                                          self.fasta_file_name,
                                                          temp_output_file,
                                                          prot_feat,
                                                          path_to_folder))
            
            
            ip = re.split('/',self.input_folder)[-1]
            if not os.path.exists("feature_extraction_output/" + ip):
            	os.makedirs("feature_extraction_output/" + ip)
            
            output_file = "feature_extraction_output/" + ip + "/{}_{}.txt".format(
                                                              self.fasta_file_name,
                                                              prot_feat)

            edit_extracted_features_POSSUM(temp_output_file, output_file, fasta_dict)

            return output_file
            
    def extract_iFeature_feature(self):
        data_iFeature = '{}/iFeature/data'.format(path_to_folder)
        if os.path.isdir(data_iFeature) == False:
            download_extract_iFeature_data()
        list_desc = [self.protein_feature]

        if self.protein_feature == 'all_iFeature':

            list_desc = self.iFeature_desc_list - {'all_iFeature'}


        for prot_feat in list_desc:
            temp_output_file = '{}/temp_folder/temp_output_folder/{}_{}.txt'.format(path_to_folder,
                                                                                    self.fasta_file_name,
                                                                                    prot_feat)
            os.system("python {}/iFeature/iFeature.py --file {}/{}.fasta --type {}" \
                      " --out {}".format(path_to_folder,
                                         self.input_folder,
                                         self.fasta_file_name,
                                         prot_feat,
                                         temp_output_file))

            ip = re.split('/',self.input_folder)[-1]
            if not os.path.isdir("feature_extraction_output/" + ip):
            	os.makedirs("feature_extraction_output/" + ip)
            
            output_file = "feature_extraction_output/" + ip + "/{}_{}.txt".format(
                                                              self.fasta_file_name,
                                                              prot_feat)


            edit_extracted_features_iFeature(temp_output_file, output_file, self.place_protein_id)

            return output_file



# -*- coding: utf-8 -*-
"""
Created on Mon Mar  8 22:32:37 2021

@author: Sameitos
"""

from profab.import_dataset import data_importer
    
class ECNO(data_importer.cls_data_loader):
    '''
    ECNO is a function to import enzyme commssion number data. It gives X data 
    and y data separately 
    Parameters:
        ratio: {None, float, list}, (default = 0.2): used to split data 
                into train, test, validation sets as given values. If left None, 
                only X and y data can be obtained while float value gives train 
                and test set. If ratio = a (float), then test will be a% of total 
                data size. If ratio = [a,b] where a and b are in (0,1), 
                train, test and validation sets are formed according to them. For example, 
                If a = 0.2 and b = 0.1, train fraction is 0.7, test fraction is 0.2 
                and validation fraction is 0.1 of all dataset size. If set_type = 'temporal', 
                then ratio = None automatically.
        protein_faeture: {'paac','aac','gaac','ctriad','ctdt','soc_number','kpssm'},
                (default = 'paac'): numerical features of protein sequences
        set_type: {'random','similarity','temporal'}, (default = 'random'):
                split type of data, random:random splitting, target:
                similarity based splitting, temporal: splitting according to
                annotation time
        pre_determined: bool, (default = False), if False, data is given
                according to ratio type, If True, already splitted data will
                be provided.
        label: {None, 'positive','negative'}, (default = None): If None, data
                is given directly, if 'negative', only negative set is given,
                If 'positive', only positive set is given.
    '''
    def __init__(self,protein_feature = 'paac',
                 set_type = 'random',
                 ratio = 0.2,
                 label = None,
                 pre_determined = True):

        super().__init__(ratio = ratio, protein_feature = protein_feature,
                         set_type = set_type,label = label,
                         pre_determined = pre_determined, main_set = 'ec_dataset') 

class GOID(data_importer.cls_data_loader):
    '''
    GOID is a function to import gene ontology term data. It gives X data and
    y data separately.
    Parameters:
        ratio: {None, float, list}, (default = 0.2): used to split data 
                into train, test, validation sets as given values. If left None, 
                only X and y data can be obtained while float value gives train 
                and test set. If ratio = a (float), then test will be a% of total 
                data size. If ratio = [a,b] where a and b are in (0,1), 
                train, test and validation sets are formed according to them. For example, 
                If a = 0.2 and b = 0.1, train fraction is 0.7, test fraction is 0.2 
                and validation fraction is 0.1 of all dataset size. If set_type = 'temporal', 
                then ratio = None automatically.
        protein_faeture: {'paac','aac','gaac','ctriad','ctdt','soc_number','kpssm'},
                (default = 'paac'): numerical features of protein sequences
        set_type: {'random','similarity','temporal'}, (default = 'random'):
                split type of data, random:random splitting, target:
                similarity based splitting, temporal: splitting according to
                annotation time
        pre_determined: bool, (default = False), if False, data is given
                according to ratio type, If True, already splitted data will
                provided.
        label: {None, 'positive','negative'}, (default = None): If None, data
                is given directly, if 'negative', only negative set is given,
                If 'positive', only positive set is given.
    '''
    def __init__(self,protein_feature = 'paac',
                 set_type = 'random',
                 ratio = 0.2,
                 label = None,
                 pre_determined = False):
        
        super().__init__(ratio = ratio, protein_feature = protein_feature,
                         set_type = set_type,label = label,
                         pre_determined = pre_determined, main_set = 'go_dataset')       

class SelfGet(data_importer.casual_importer):
    
    '''
    Description:
        This function is to provide users to import their datasets with
        specified delimiter. The format of data should be like that if 
        delimiter is comma separated and name == True:
            
            Name(or ID),feature_1,feature_2,...,feature_n
            Name(or ID),feature_1,feature_2,...,feature_n
            Name(or ID),feature_1,feature_2,...,feature_n
        
    Parameters:
        delimiter: default = "\t", a character to separate columns in file.
        name: type = bool, default = False, If True, then first colmun
            is considered as name of inputs else the first column is a 
            feature column.
        label: type = bool, default = False, If True, then last colmun
            is considered as label of inputs else the last column is a 
            feature column. 
        
    '''
    
    def __init__(self, delimiter = '\t', name = False, label = False):
        
        super().__init__(delimiter = delimiter, name = name, label = label)



"""!!!!THIS CLASS IS OUT OF USE FOR NOW. AS DTI DATASETS ARE IN PIPE,
    DTI IMPORTER WILL BE READY TO USE!!!!"""
# class DTI(data_importer.rgs_data_loader):
    
#     '''
#     DTI is a function to import drug-target interaction data. It gives X data 
#     and y data separately
#     Parameters:
#         ratio: {None, float, list}, (default = 0.2): used to split data 
#                 into train, test, validation sets as given values. If left None, 
#                 only X and y data can be obtained while float value gives train 
#                 and test set. If ratio = a (float), then test will be a% of total 
#                 data size. If ratio = [a,b] where a and b are in (0,1), train, test 
#                 and validation sets are formed according to them. For example, 
#                 If a = 0.2 and b = 0.1, train fraction is 0.7, test fraction is 0.2 
#                 and validation fraction is 0.1 of all dataset size.
#         protein_faeture: {'paac','aac','gaac','ctriad','ctdt','soc_number', kpssm},
#                 (default = 'paac'): numerical features of protein sequences
#         set_type: {'random','target'}, (default = 'random'):
#                 split type of data, random:random splitting, target:
#                 similarity based splitting
#     '''
#     def __init__(self,protein_feature = 'paac',
#                  set_type = 'random',
#                  ratio = None):
        
#         self.protein_feature = protein_feature
#         self.set_type = set_type
#         self.ratio = ratio
#         super().__init__(ratio = self.ratio, protein_feature = self.protein_feature,
#                          set_type = self.set_type)


#!/usr/bin/env python
#_*_coding:utf-8_*_

from sklearn.cluster import AffinityPropagation
import numpy as np

def apc(encodings, **kw):
	if kw['sof'] == 'sample':
		encodings = np.array(encodings)[1:]
	else:
		encodings = np.array(encodings).T[1:]
	#encodings = np.array(encodings)[1:]

	if len(encodings) < 3:
		return 0, 'sample number should be greater than 3.'

	data = encodings[:, 1:]
	shape = data.shape
	data = np.reshape(data, shape[0] * shape[1])
	data = np.reshape([float(i) for i in data], shape)

	e = ''
	try:
		af = AffinityPropagation().fit(data)
	except ValueError as e:
		print(e)
		return 0, e

	cluster_centers_indices = af.cluster_centers_indices_
	labels = af.labels_
	#n_clusters_ = len(cluster_centers_indices)

	res = []
	for i in range(len(data)):
		res.append([encodings[i][0],labels[i]])
	return res, 0